@@ -19,7 +19,8 @@
 
 import React from 'react';
 import PropTypes from 'prop-types';
-import { Button, FormGroup, HelpBlock, Modal, OverlayTrigger, Tooltip, TypeAheadSelect } from 'patternfly-react';
+import { FormGroup, HelpBlock, Modal, OverlayTrigger, Tooltip, TypeAheadSelect } from 'patternfly-react';
+import { Button } from '@patternfly/react-core';
 
 import cockpit from 'cockpit';
 import { MachinesConnectionSelector } from '../machinesConnectionSelector.jsx';
@@ -50,15 +51,18 @@ import {
     getOSStringRepresentation,
 } from "./createVmDialogUtils.js";
 import MemorySelectRow from '../memorySelectRow.jsx';
+import { storagePoolRefresh } from '../../libvirt-dbus.js';
+import { Password } from './password.jsx';
 
-import './createVmDialog.less';
-import 'form-layout.less';
+import './createVmDialog.scss';
+import 'form-layout.scss';
 import VMS_CONFIG from '../../config.js';
 
 const _ = cockpit.gettext;
 
 const URL_SOURCE = 'url';
 const LOCAL_INSTALL_MEDIA_SOURCE = 'file';
+const DOWNLOAD_AN_OS = 'os';
 const EXISTING_DISK_IMAGE_SOURCE = 'disk_image';
 const PXE_SOURCE = 'pxe';
 
@@ -117,9 +121,10 @@ function getSpaceAvailable(storagePools, connectionName) {
 function validateParams(vmParams) {
     const validationFailed = {};
 
-    if (isEmpty(vmParams.vmName.trim())) {
-        validationFailed.vmName = _("Name should not be empty");
-    }
+    if (isEmpty(vmParams.vmName.trim()))
+        validationFailed.vmName = _("Name must not be empty");
+    else if (vmParams.vms.some(vm => vm.name === vmParams.vmName))
+        validationFailed.vmName = cockpit.format(_("VM $0 already exists"), vmParams.vmName);
 
     if (vmParams.os == undefined)
         validationFailed.os = _("You need to select the most closely matching Operating System");
@@ -145,8 +150,8 @@ function validateParams(vmParams) {
             }
             break;
         }
-    } else {
-        validationFailed.source = _("Installation Source should not be empty");
+    } else if (vmParams.sourceType != DOWNLOAD_AN_OS) {
+        validationFailed.source = _("Installation Source must not be empty");
     }
 
     if (vmParams.memorySize === 0) {
@@ -164,7 +169,7 @@ function validateParams(vmParams) {
         }
     }
 
-    if (vmParams.storagePool === "NewVolume") {
+    if (vmParams.sourceType != EXISTING_DISK_IMAGE_SOURCE && vmParams.storagePool === "NewVolume") {
         if (vmParams.storageSize === 0) {
             validationFailed.storage = _("Storage size must not be 0");
         } else if (vmParams.os &&
@@ -178,6 +183,8 @@ function validateParams(vmParams) {
             );
         }
     }
+    if (vmParams.unattendedInstallation && !vmParams.rootPassword && !vmParams.userPassword)
+        validationFailed.password = _("Please set a root or a user password");
 
     return validationFailed;
 }
@@ -206,7 +213,7 @@ const NameRow = ({ vmName, onValueChanged, validationFailed }) => {
     );
 };
 
-const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices, providerName, onValueChanged, validationFailed }) => {
+const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices, providerName, os, osInfoList, downloadOSSupported, onValueChanged, validationFailed }) => {
     let installationSource;
     let installationSourceId;
     let installationSourceWarning;
@@ -259,7 +266,6 @@ const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices,
         );
         break;
     case URL_SOURCE:
-    default:
         installationSourceId = "source-url";
         installationSource = (
             <input id={installationSourceId} className="form-control"
@@ -270,6 +276,8 @@ const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices,
                 onChange={e => onValueChanged('source', e.target.value)} />
         );
         break;
+    default:
+        break;
     }
 
     return (
@@ -277,11 +285,13 @@ const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices,
             {sourceType != EXISTING_DISK_IMAGE_SOURCE &&
             <>
                 <label className="control-label" htmlFor="source-type">
-                    {_("Installation Source Type")}
+                    {_("Installation Type")}
                 </label>
                 <Select.Select id="source-type"
                     initial={sourceType}
                     onChange={value => onValueChanged('sourceType', value)}>
+                    {downloadOSSupported ? <Select.SelectEntry data={DOWNLOAD_AN_OS}
+                        key={DOWNLOAD_AN_OS}>{_("Download an OS")}</Select.SelectEntry> : null}
                     <Select.SelectEntry data={LOCAL_INSTALL_MEDIA_SOURCE}
                         key={LOCAL_INSTALL_MEDIA_SOURCE}>{_("Local Install Media")}</Select.SelectEntry>
                     <Select.SelectEntry data={URL_SOURCE} key={URL_SOURCE}>{_("URL")}</Select.SelectEntry>
@@ -294,16 +304,24 @@ const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices,
                 </Select.Select>
             </>}
 
-            <label className="control-label" htmlFor={installationSourceId}>
-                {_("Installation Source")}
-            </label>
-            <FormGroup validationState={validationStateSource} controlId='source'>
-                {installationSource}
-                { validationStateSource == 'error' &&
-                <HelpBlock>
-                    <p className="text-danger">{validationFailed.source}</p>
-                </HelpBlock> }
-            </FormGroup>
+            {sourceType != DOWNLOAD_AN_OS
+                ? <>
+                    <label className="control-label" htmlFor={installationSourceId}>
+                        {_("Installation Source")}
+                    </label>
+                    <FormGroup validationState={validationStateSource} controlId='source'>
+                        {installationSource}
+                        { validationStateSource == 'error' &&
+                        <HelpBlock>
+                            <p className="text-danger">{validationFailed.source}</p>
+                        </HelpBlock> }
+                    </FormGroup>
+                </>
+                : <OSRow os={os}
+                         osInfoList={osInfoList.filter(os => os.treeInstallable)}
+                         onValueChanged={onValueChanged}
+                         isLoading={false}
+                         validationFailed={validationFailed} />}
         </>
     );
 };
@@ -374,15 +392,80 @@ class OSRow extends React.Component {
     }
 }
 
-const MemoryRow = ({ memorySize, memorySizeUnit, nodeMaxMemory, recommendedMemory, onValueChanged, validationFailed }) => {
-    const validationStateMemory = validationFailed.memory ? 'error' : undefined;
-    let recommendedMemoryHelpBlock = null;
-    if (recommendedMemory && recommendedMemory > memorySize) {
-        recommendedMemoryHelpBlock = <p>{cockpit.format(
-            _("The selected Operating System has recommended memory $0 $1"),
-            recommendedMemory, memorySizeUnit)}</p>;
+const UnattendedRow = ({ validationFailed, unattendedDisabled, unattendedInstallation, os, profile, onValueChanged }) => {
+    const validationStatePassword = validationFailed.password ? 'error' : undefined;
+    let unattendedInstallationCheckbox = (
+        <label className="checkbox-inline">
+            <input id="unattended-installation" type="checkbox"
+                checked={unattendedInstallation}
+                disabled={unattendedDisabled}
+                onChange={e => onValueChanged('unattendedInstallation', e.target.checked)} />
+            {_("Run unattended installation")}
+        </label>
+    );
+    if (unattendedDisabled) {
+        unattendedInstallationCheckbox = (
+            <OverlayTrigger overlay={ <Tooltip id='os-unattended-installation-tooltip'>{ _("The selected Operating System does not support unattended installation") }</Tooltip> } placement='left'>
+                {unattendedInstallationCheckbox}
+            </OverlayTrigger>
+        );
     }
 
+    return (
+        <>
+            {unattendedInstallationCheckbox}
+            {!unattendedDisabled && unattendedInstallation ? <>
+                {os.profiles.length > 0 && <>
+                    <label className="control-label" htmlFor="profile-select">
+                        {_("Profile")}
+                    </label>
+                    <Select.Select id="profile-select"
+                        initial={os.profiles && os.profiles[0]}
+                        onChange={e => onValueChanged('profile', e)}>
+                        { (os.profiles || []).sort()
+                                .reverse() // Let jeos (Server) appear always first on the list since in osinfo-db it's not consistent
+                                .map(profile => {
+                                    let profileName;
+                                    if (profile == 'jeos')
+                                        profileName = 'Server';
+                                    else if (profile == 'desktop')
+                                        profileName = 'Workstation';
+                                    else
+                                        profileName = profile;
+                                    return <Select.SelectEntry data={profile} key={profile}>{profileName}</Select.SelectEntry>;
+                                }) }
+                    </Select.Select>
+                </>}
+                <label htmlFor='root-password' className='control-label'>
+                    {_("Root Password")}
+                </label>
+                <FormGroup validationState={validationStatePassword} bsClass='form-group ct-validation-wrapper' controlId='root-password'>
+                    <Password id='root-password' onValueChanged={(value) => onValueChanged('rootPassword', value)} />
+                    <HelpBlock>
+                        <p className="text-danger">{validationFailed.password}</p>
+                        {profile == 'desktop' && <p className="text-info">{_("Leave the password blank if you do not wish to have a root account created")}</p>}
+                    </HelpBlock>
+                </FormGroup>
+                {profile == 'desktop' && <>
+                    <label htmlFor='user-password' className='control-label'>
+                        {cockpit.format(_("Password for $0"), permission.user.name)}
+                    </label>
+                    <FormGroup validationState={validationStatePassword} bsClass='form-group ct-validation-wrapper' controlId='user-password'>
+                        <Password id='user-password' onValueChanged={(value) => onValueChanged('userPassword', value)} />
+                        <HelpBlock>
+                            <p className="text-danger">{validationFailed.password}</p>
+                            <p className="text-info">{_("Leave the password blank if you do not wish to have a user account created")}</p>
+                        </HelpBlock>
+                    </FormGroup>
+                </>}
+                <hr />
+            </> : <span />}
+        </>
+    );
+};
+
+const MemoryRow = ({ memorySize, memorySizeUnit, nodeMaxMemory, recommendedMemory, minimumMemory, onValueChanged, validationFailed }) => {
+    const validationStateMemory = validationFailed.memory ? 'error' : undefined;
     return (
         <>
             <label htmlFor='memory-size' className='control-label'>
@@ -390,34 +473,22 @@ const MemoryRow = ({ memorySize, memorySizeUnit, nodeMaxMemory, recommendedMemor
             </label>
             <FormGroup validationState={validationStateMemory} bsClass='form-group ct-validation-wrapper' controlId='memory'>
                 <MemorySelectRow id='memory-size'
-                    value={memorySize}
-                    maxValue={nodeMaxMemory && convertToUnit(nodeMaxMemory, units.KiB, memorySizeUnit)}
+                    value={Math.max(memorySize, Math.floor(convertToUnit(minimumMemory, units.B, memorySizeUnit)))}
+                    maxValue={nodeMaxMemory && Math.floor(convertToUnit(nodeMaxMemory, units.KiB, memorySizeUnit))}
+                    minValue={Math.floor(convertToUnit(minimumMemory, units.B, memorySizeUnit))}
                     initialUnit={memorySizeUnit}
-                    onValueChange={e => onValueChanged('memorySize', e.target.value)}
+                    onValueChange={value => onValueChanged('memorySize', value)}
                     onUnitChange={value => onValueChanged('memorySizeUnit', value)} />
                 <HelpBlock id="memory-size-helpblock">
                     {validationStateMemory === "error" && <p>{validationFailed.memory}</p>}
-                    {recommendedMemoryHelpBlock}
-                    {nodeMaxMemory && <p> {cockpit.format(
-                        _("Up to $0 $1 available on the host"),
-                        Math.floor(convertToUnit(nodeMaxMemory, units.KiB, memorySizeUnit)),
-                        memorySizeUnit,
-                    )}</p>}
                 </HelpBlock>
             </FormGroup>
         </>
     );
 };
 
-const StorageRow = ({ connectionName, storageSize, storageSizeUnit, onValueChanged, recommendedStorage, storagePoolName, storagePools, storageVolume, vms, validationFailed }) => {
+const StorageRow = ({ connectionName, storageSize, storageSizeUnit, onValueChanged, recommendedStorage, minimumStorage, storagePoolName, storagePools, storageVolume, vms, validationFailed }) => {
     const validationStateStorage = validationFailed.storage ? 'error' : undefined;
-    let recommendedStorageHelpBlock = null;
-    if (recommendedStorage && recommendedStorage > storageSize) {
-        recommendedStorageHelpBlock = <p>{cockpit.format(
-            _("The selected Operating System has recommended storage size of $0 $1"),
-            recommendedStorage, storageSizeUnit)}</p>;
-    }
-
     let volumeEntries;
     let isVolumeUsed = {};
     // Existing storage pool is chosen
@@ -476,20 +547,15 @@ const StorageRow = ({ connectionName, storageSize, storageSizeUnit, onValueChang
                 </label>
                 <FormGroup validationState={validationStateStorage} bsClass='form-group ct-validation-wrapper' controlId='storage'>
                     <MemorySelectRow id="storage-size"
-                        value={storageSize}
-                        maxValue={poolSpaceAvailable && convertToUnit(poolSpaceAvailable, units.B, storageSizeUnit)}
+                        value={Math.max(storageSize, Math.floor(convertToUnit(minimumStorage || 0, units.B, storageSizeUnit)))}
+                        maxValue={poolSpaceAvailable && Math.floor(convertToUnit(poolSpaceAvailable, units.B, storageSizeUnit))}
+                        minValue={minimumStorage && Math.floor(convertToUnit(minimumStorage, units.B, storageSizeUnit))}
                         initialUnit={storageSizeUnit}
-                        onValueChange={e => onValueChanged('storageSize', e.target.value)}
+                        onValueChange={value => onValueChanged('storageSize', value)}
                         onUnitChange={value => onValueChanged('storageSizeUnit', value)} />
                     {poolSpaceAvailable &&
                     <HelpBlock id="storage-size-helpblock">
                         {validationStateStorage === "error" && <p>{validationFailed.storage}</p>}
-                        {recommendedStorageHelpBlock}
-                        {cockpit.format(
-                            _("Up to $0 $1 available in the default location"),
-                            Math.floor(convertToUnit(poolSpaceAvailable, units.B, storageSizeUnit)),
-                            storageSizeUnit,
-                        )}
                     </HelpBlock>}
                 </FormGroup>
             </> }
@@ -499,25 +565,40 @@ const StorageRow = ({ connectionName, storageSize, storageSizeUnit, onValueChang
 
 class CreateVmModal extends React.Component {
     constructor(props) {
+        let defaultSourceType;
+        if (props.mode == 'create') {
+            if (!props.downloadOSSupported)
+                defaultSourceType = LOCAL_INSTALL_MEDIA_SOURCE;
+            else
+                defaultSourceType = DOWNLOAD_AN_OS;
+        } else {
+            defaultSourceType = EXISTING_DISK_IMAGE_SOURCE;
+        }
         super(props);
         this.state = {
             inProgress: false,
             validate: false,
             vmName: '',
             connectionName: LIBVIRT_SYSTEM_CONNECTION,
-            sourceType: props.mode == 'create' ? LOCAL_INSTALL_MEDIA_SOURCE : EXISTING_DISK_IMAGE_SOURCE,
+            sourceType: defaultSourceType,
+            unattendedInstallation: false,
             source: '',
             os: undefined,
             memorySize: Math.min(convertToUnit(1024, units.MiB, units.GiB), // tied to Unit
                                  Math.floor(convertToUnit(props.nodeMaxMemory, units.KiB, units.GiB))),
             memorySizeUnit: units.GiB.name,
-            storageSize: 10, // GiB
+            storageSize: Math.min(convertToUnit(10 * 1024, units.MiB, units.GiB), // tied to Unit
+                                  Math.floor(convertToUnit(props.nodeMaxMemory, units.KiB, units.GiB))),
             storageSizeUnit: units.GiB.name,
             storagePool: 'NewVolume',
             storageVolume: '',
-            startVm: false,
+            startVm: true,
             recommendedMemory: undefined,
+            minimumMemory: 0,
             recommendedStorage: undefined,
+            minimumStorage: 0,
+            userPassword: '',
+            rootPassword: '',
         };
         this.onCreateClicked = this.onCreateClicked.bind(this);
         this.onValueChanged = this.onValueChanged.bind(this);
@@ -525,6 +606,9 @@ class CreateVmModal extends React.Component {
 
     onValueChanged(key, value) {
         switch (key) {
+        case 'vmName':
+            this.setState({ [key]: value.split(" ").join("_") });
+            break;
         case 'source':
             this.setState({ [key]: value });
 
@@ -535,14 +619,16 @@ class CreateVmModal extends React.Component {
                 const onOsAutodetect = (os) => {
                     this.setState({ autodetectOSInProgress: true });
                     autodetectOS(os)
-                            .then(res => {
-                                const osEntry = this.props.osInfoList.filter(osEntry => osEntry.id == res);
+                            .then(resJSON => {
+                                const res = JSON.parse(resJSON);
+                                const osEntry = this.props.osInfoList.filter(osEntry => osEntry.id == res.os);
 
                                 if (osEntry && osEntry[0]) {
                                     this.setState({
                                         autodetectOSInProgress: false,
                                     });
                                     this.onValueChanged('os', osEntry[0]);
+                                    this.onValueChanged('sourceMediaID', res.media);
                                 }
                             }, ex => {
                                 console.log("osinfo-detect command failed: ", ex.message);
@@ -629,21 +715,46 @@ class CreateVmModal extends React.Component {
             }
             break;
         case 'os': {
-            this.setState(prevState => { // to prevent asynchronous calls
-                const stateDelta = { [key]: value };
-                if (value && value.recommendedResources.ram)
-                    stateDelta.recommendedMemory = convertToUnit(value.recommendedResources.ram, units.B, prevState.memorySizeUnit);
-                else
-                    stateDelta.recommendedMemory = undefined;
-                if (value && value.recommendedResources.storage)
-                    stateDelta.recommendedStorage = convertToUnit(value.recommendedResources.storage, units.B, prevState.storageSizeUnit);
-                else
-                    stateDelta.recommendedStorage = undefined;
+            const stateDelta = { [key]: value };
 
-                return stateDelta;
-            });
+            if (value && value.minimumResources.ram)
+                stateDelta.minimumMemory = value.minimumResources.ram;
+
+            if (value && value.profiles)
+                stateDelta.profile = value.profiles.sort().reverse()[0];
+
+            if (value && value.recommendedResources.ram) {
+                stateDelta.recommendedMemory = value.recommendedResources.ram;
+                const converted = Math.floor(convertToUnit(stateDelta.recommendedMemory, units.B, this.state.memorySizeUnit));
+                if (converted == 0)
+                    this.setState({ memorySizeUnit: units.MiB.name, memorySize: Math.floor(convertToUnit(stateDelta.recommendedMemory, units.B, units.MiB)) });
+                else
+                    this.onValueChanged('memorySize', converted);
+            } else {
+                stateDelta.recommendedMemory = undefined;
+            }
+
+            if (value && value.minimumResources.storage)
+                stateDelta.minimumStorage = value.minimumResources.storage;
+
+            if (value && value.recommendedResources.storage) {
+                stateDelta.recommendedStorage = value.recommendedResources.storage;
+                const converted = Math.floor(convertToUnit(stateDelta.recommendedStorage, units.B, this.state.storageSizeUnit));
+                if (converted == 0)
+                    this.setState({ storageSizeUnit: units.MiB.name, storageSize: Math.floor(convertToUnit(stateDelta.recommendedStorage, units.B, units.MiB)) });
+                else
+                    this.onValueChanged('storageSize', converted);
+            } else {
+                stateDelta.recommendedStorage = undefined;
+            }
+            if (!value.unattendedInstallable)
+                this.onValueChanged('unattendedInstallation', false);
+            this.setState(stateDelta);
             break;
         }
+        case 'unattendedInstallation':
+            this.setState({ unattendedInstallation: value, startVm: true });
+            break;
         default:
             this.setState({ [key]: value });
             break;
@@ -651,15 +762,16 @@ class CreateVmModal extends React.Component {
     }
 
     onCreateClicked() {
-        const { dispatch } = this.props;
+        const { dispatch, providerName, storagePools, close, onAddErrorNotification, osInfoList, vms } = this.props;
 
-        if (Object.getOwnPropertyNames(validateParams({ ...this.state, osInfoList: this.props.osInfoList })).length > 0) {
+        const validation = validateParams({ ...this.state, osInfoList, vms: vms.filter(vm => vm.connectionName == this.state.connectionName) });
+        if (Object.getOwnPropertyNames(validation).length > 0) {
             this.setState({ inProgress: false, validate: true });
         } else {
             // leave dialog open to show immediate errors from the backend
             // close the dialog after VMS_CONFIG.LeaveCreateVmDialogVisibleAfterSubmit
             // then show errors in the notification area
-            this.setState({ inProgress: true });
+            this.setState({ inProgress: true, validate: false });
 
             const vmParams = {
                 connectionName: this.state.connectionName,
@@ -667,40 +779,71 @@ class CreateVmModal extends React.Component {
                 source: this.state.source,
                 sourceType: this.state.sourceType,
                 os: this.state.os ? this.state.os.shortId : 'auto',
+                profile: this.state.profile,
                 memorySize: convertToUnit(this.state.memorySize, this.state.memorySizeUnit, units.MiB),
                 storageSize: convertToUnit(this.state.storageSize, this.state.storageSizeUnit, units.GiB),
                 storagePool: this.state.storagePool,
                 storageVolume: this.state.storageVolume,
                 startVm: this.state.startVm,
+                unattended: this.state.unattendedInstallation,
+                userPassword: this.state.userPassword,
+                rootPassword: this.state.rootPassword,
             };
 
             return timeoutedPromise(
                 dispatch(createVm(vmParams)),
                 VMS_CONFIG.LeaveCreateVmDialogVisibleAfterSubmit,
-                () => this.props.close(),
+                () => {
+                    close();
+
+                    if (providerName == 'LibvirtDBus' && this.state.storagePool === "NewVolume") {
+                        const storagePool = storagePools.find(pool => pool.connectionName === this.state.connectionName && pool.name === "default");
+                        if (storagePool)
+                            storagePoolRefresh(storagePool.connectionName, storagePool.id);
+                    }
+                },
                 (exception) => {
-                    this.props.onAddErrorNotification({
+                    onAddErrorNotification({
                         text: cockpit.format(_("Creation of VM $0 failed"), vmParams.vmName),
                         detail: exception.message,
                     });
-                    this.props.close();
+                    close();
                 });
         }
     }
 
     render() {
         const { nodeMaxMemory, nodeDevices, networks, osInfoList, loggedUser, providerName, storagePools, vms } = this.props;
-        const validationFailed = this.state.validate && validateParams({ ...this.state, osInfoList });
+        const validationFailed = this.state.validate && validateParams({ ...this.state, osInfoList, vms: vms.filter(vm => vm.connectionName == this.state.connectionName) });
+        let startVmCheckbox = (
+            <label className="checkbox-inline">
+                <input id="start-vm" type="checkbox"
+                    checked={this.state.startVm}
+                    disabled={this.state.unattendedInstallation}
+                    onChange={e => this.onValueChanged('startVm', e.target.checked)} />
+                {_("Immediately Start VM")}
+            </label>
+        );
+        if (this.state.unattendedInstallation) {
+            startVmCheckbox = (
+                <OverlayTrigger overlay={ <Tooltip id='virt-install-not-available-tooltip'>{ _("Setting the user passwords for unattended installation requires starting the VM when creating it") }</Tooltip> } placement='left'>
+                    {startVmCheckbox}
+                </OverlayTrigger>
+            );
+        }
+
+        let unattendedDisabled = true;
+        if ((this.state.sourceType == URL_SOURCE || this.state.sourceType == LOCAL_INSTALL_MEDIA_SOURCE) && this.state.os) {
+            if (this.state.os.medias && this.state.sourceMediaID in this.state.os.medias)
+                unattendedDisabled = !this.state.os.medias[this.state.sourceMediaID].unattendedInstallable;
+            else
+                unattendedDisabled = !this.state.os.unattendedInstallable;
+        } else if (this.state.sourceType == DOWNLOAD_AN_OS) {
+            unattendedDisabled = !this.state.os || !this.state.os.unattendedInstallable;
+        }
 
         const dialogBody = (
             <form className="ct-form">
-                <MachinesConnectionSelector id='connection'
-                    connectionName={this.state.connectionName}
-                    onValueChanged={this.onValueChanged}
-                    loggedUser={loggedUser} />
-
-                <hr />
-
                 <NameRow
                     vmName={this.state.vmName}
                     onValueChanged={this.onValueChanged}
@@ -708,6 +851,13 @@ class CreateVmModal extends React.Component {
 
                 <hr />
 
+                <MachinesConnectionSelector id='connection'
+                    connectionName={this.state.connectionName}
+                    onValueChanged={this.onValueChanged}
+                    loggedUser={loggedUser} />
+
+                <hr />
+
                 <SourceRow
                     connectionName={this.state.connectionName}
                     networks={networks.filter(network => network.connectionName == this.state.connectionName)}
@@ -715,28 +865,41 @@ class CreateVmModal extends React.Component {
                     providerName={providerName}
                     source={this.state.source}
                     sourceType={this.state.sourceType}
+                    os={this.state.os}
+                    osInfoList={this.props.osInfoList}
+                    downloadOSSupported={this.props.downloadOSSupported}
                     onValueChanged={this.onValueChanged}
                     validationFailed={validationFailed} />
 
                 <hr />
 
-                { this.state.sourceType != EXISTING_DISK_IMAGE_SOURCE &&
+                {this.state.sourceType != DOWNLOAD_AN_OS &&
                 <>
-                    <StorageRow
-                        connectionName={this.state.connectionName}
-                        storageSize={this.state.storageSize}
-                        storageSizeUnit={this.state.storageSizeUnit}
+                    <OSRow
+                        os={this.state.os}
+                        osInfoList={this.props.osInfoList}
                         onValueChanged={this.onValueChanged}
-                        storagePoolName={this.state.storagePool}
-                        storagePools={storagePools.filter(pool => pool.connectionName === this.state.connectionName)}
-                        storageVolume={this.state.storageVolume}
-                        vms={vms}
-                        recommendedStorage={this.state.recommendedStorage}
-                        validationFailed={validationFailed}
-                    />
+                        isLoading={this.state.autodetectOSInProgress}
+                        validationFailed={validationFailed} />
+
                     <hr />
                 </>}
 
+                { this.state.sourceType != EXISTING_DISK_IMAGE_SOURCE &&
+                <StorageRow
+                    connectionName={this.state.connectionName}
+                    storageSize={this.state.storageSize}
+                    storageSizeUnit={this.state.storageSizeUnit}
+                    onValueChanged={this.onValueChanged}
+                    storagePoolName={this.state.storagePool}
+                    storagePools={storagePools.filter(pool => pool.connectionName === this.state.connectionName)}
+                    storageVolume={this.state.storageVolume}
+                    vms={vms}
+                    recommendedStorage={this.state.recommendedStorage}
+                    minimumStorage={this.state.minimumStorage}
+                    validationFailed={validationFailed}
+                />}
+
                 <MemoryRow
                     memorySize={this.state.memorySize}
                     memorySizeUnit={this.state.memorySizeUnit}
@@ -744,25 +907,26 @@ class CreateVmModal extends React.Component {
                     onValueChanged={this.onValueChanged}
                     validationFailed={validationFailed}
                     recommendedMemory={this.state.recommendedMemory}
+                    minimumMemory={this.state.minimumMemory}
                 />
 
                 <hr />
 
-                <OSRow
-                    os={this.state.os}
-                    osInfoList={this.props.osInfoList}
-                    onValueChanged={this.onValueChanged}
-                    isLoading={this.state.autodetectOSInProgress}
-                    validationFailed={validationFailed} />
+                {this.state.sourceType != PXE_SOURCE &&
+                 this.state.sourceType != EXISTING_DISK_IMAGE_SOURCE &&
+                 this.props.unattendedSupported &&
+                 <>
+                     <UnattendedRow
+                         validationFailed={validationFailed}
+                         unattendedDisabled={unattendedDisabled}
+                         unattendedInstallation={this.state.unattendedInstallation}
+                         os={this.state.os}
+                         profile={this.state.profile}
+                         onValueChanged={this.onValueChanged} />
+                     <hr />
+                 </>}
 
-                <hr />
-
-                <label className="checkbox-inline">
-                    <input id="start-vm" type="checkbox"
-                        checked={this.state.startVm}
-                        onChange={e => this.onValueChanged('startVm', e.target.checked)} />
-                    {_("Immediately Start VM")}
-                </label>
+                {startVmCheckbox}
             </form>
         );
 
@@ -770,20 +934,20 @@ class CreateVmModal extends React.Component {
             <Modal id='create-vm-dialog' show onHide={ this.props.close }>
                 <Modal.Header>
                     <Modal.CloseButton onClick={ this.props.close } />
-                    <Modal.Title> {`Create New Virtual Machine`} </Modal.Title>
+                    <Modal.Title> {this.props.mode == 'create' ? _("Create New Virtual Machine") : _("Import A Virtual Machine")} </Modal.Title>
                 </Modal.Header>
                 <Modal.Body>
                     {dialogBody}
                 </Modal.Body>
                 <Modal.Footer>
                     {this.state.inProgress && <div className="spinner spinner-sm pull-left" />}
-                    <Button bsStyle='default' className='btn-cancel' onClick={ this.props.close }>
+                    <Button variant='secondary' className='btn-cancel' onClick={ this.props.close }>
                         {_("Cancel")}
                     </Button>
-                    <Button bsStyle='primary'
-                            disabled={Object.getOwnPropertyNames(validationFailed).length > 0}
+                    <Button variant="primary"
+                            isDisabled={Object.getOwnPropertyNames(validationFailed).length > 0}
                             onClick={this.onCreateClicked}>
-                        {_("Create")}
+                        {this.props.mode == 'create' ? _("Create") : _("Import")}
                     </Button>
                 </Modal.Footer>
             </Modal>
@@ -794,16 +958,29 @@ class CreateVmModal extends React.Component {
 export class CreateVmAction extends React.Component {
     constructor(props) {
         super(props);
-        this.state = { showModal: false };
+        this.state = {
+            showModal: false,
+            virtInstallAvailable: undefined,
+            downloadOSSupported: undefined,
+            unattendedSupported: undefined
+        };
         this.open = this.open.bind(this);
         this.close = this.close.bind(this);
-        this.state = { virtInstallAvailable: undefined };
     }
 
     componentDidMount() {
         cockpit.spawn(['which', 'virt-install'], { err: 'message' })
-                .then(() => this.setState({ virtInstallAvailable: true })
-                    , () => this.setState({ virtInstallAvailable: false }));
+                .then(() => {
+                    this.setState({ virtInstallAvailable: true });
+                    cockpit.spawn(['virt-install', '--install=?'])
+                            .then(() => this.setState({ downloadOSSupported: true }),
+                                  () => this.setState({ downloadOSSupported: false }));
+
+                    cockpit.spawn(['virt-install', '--unattended=?'])
+                            .then(() => this.setState({ unattendedSupported: true }),
+                                  () => this.setState({ unattendedSupported: false }));
+                },
+                      () => this.setState({ virtInstallAvailable: false }));
     }
 
     // That will stop any state setting on unmounted/unmounting components
@@ -820,21 +997,38 @@ export class CreateVmAction extends React.Component {
     }
 
     render() {
-        if (this.state.virtInstallAvailable == undefined)
+        if (this.props.systemInfo.osInfoList == null)
             return null;
 
+        let testdata;
+        if (!this.props.systemInfo.osInfoList)
+            testdata = "disabledOsInfo";
+        else if (!this.state.virtInstallAvailable)
+            testdata = "disabledVirtInstall";
+        else if (this.state.downloadOSSupported === undefined)
+            testdata = "disabledDownloadOS";
         let createButton = (
-            <Button disabled={!this.props.systemInfo.osInfoList || !this.state.virtInstallAvailable}
+            <Button isDisabled={!(this.props.systemInfo.osInfoList &&
+                               (this.state.virtInstallAvailable !== undefined ||
+                                   (this.state.virtInstallAvailable &&
+                                       (this.state.downloadOSSupported === undefined || this.state.unattendedSupported === undefined))))}
+                    testdata={testdata}
                     id={this.props.mode == 'create' ? 'create-new-vm' : 'import-vm-disk'}
-                    bsStyle='default'
+                    variant='secondary'
+                    style={!this.state.virtInstallAvailable ? { pointerEvents: 'none' } : null} // Fixes OverlayTrigger not showing up
                     onClick={this.open}>
                 {this.props.mode == 'create' ? _("Create VM") : _("Import VM")}
             </Button>
         );
         if (!this.state.virtInstallAvailable)
             createButton = (
-                <OverlayTrigger overlay={ <Tooltip id='virt-install-not-available-tooltip'>{ _("virt-install package needs to be installed on the system in order to create new VMs") }</Tooltip> } placement='top'>
-                    {createButton}
+                <OverlayTrigger overlay={
+                    <Tooltip id='virt-install-not-available-tooltip'>
+                        {_("virt-install package needs to be installed on the system in order to create new VMs")}
+                    </Tooltip>} placement='top'>
+                    <span>
+                        {createButton}
+                    </span>
                 </OverlayTrigger>
             );
 
@@ -849,10 +1043,14 @@ export class CreateVmAction extends React.Component {
                     networks={this.props.networks}
                     nodeDevices={this.props.nodeDevices}
                     nodeMaxMemory={this.props.nodeMaxMemory}
-                    storagePools={this.props.storagePools}
+                    // The initial resources fetching contains only ID - this will be immediately
+                    // replaced with the whole resource object but there is enough time to cause a crash if parsed here
+                    storagePools={this.props.storagePools.filter(pool => pool.name)}
                     vms={this.props.vms}
                     osInfoList={this.props.systemInfo.osInfoList}
                     onAddErrorNotification={this.props.onAddErrorNotification}
+                    downloadOSSupported={this.state.downloadOSSupported}
+                    unattendedSupported={this.state.unattendedSupported}
                     loggedUser={this.props.systemInfo.loggedUser} /> }
             </>
         );
