@@ -17,16 +17,18 @@
  * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
  */
 import React from 'react';
-import { Button, Modal } from 'patternfly-react';
+import { Modal } from 'patternfly-react';
+import { Button } from '@patternfly/react-core';
 import cockpit from 'cockpit';
 
 import * as Select from "cockpit-components-select.jsx";
 import { ModalError } from 'cockpit-components-inline-notification.jsx';
-import { units, convertToUnit, getDefaultVolumeFormat } from '../helpers.js';
-import { volumeCreateAndAttach, attachDisk, getVm, getAllStoragePools } from '../actions/provider-actions.js';
+import { units, convertToUnit, getDefaultVolumeFormat, getNextAvailableTarget, getStorageVolumesUsage, getStorageVolumeDiskTarget } from '../helpers.js';
+import { volumeCreateAndAttach, attachDisk, getVm } from '../actions/provider-actions.js';
 import { VolumeCreateBody } from './storagePools/storageVolumeCreateBody.jsx';
+import { updateDiskAttributes } from '../libvirt-dbus.js';
 
-import 'form-layout.less';
+import 'form-layout.scss';
 import './diskAdd.css';
 
 const _ = cockpit.gettext;
@@ -34,18 +36,6 @@ const _ = cockpit.gettext;
 const CREATE_NEW = 'create-new';
 const USE_EXISTING = 'use-existing';
 
-function getNextAvailableTarget(vm) {
-    const existingTargets = Object.getOwnPropertyNames(vm.disks);
-    const targets = [];
-    let i = 0;
-    while (i < 26 && targets.length < 5) {
-        const target = `vd${String.fromCharCode(97 + i)}`;
-        if (!existingTargets.includes(target))
-            return target;
-        i++;
-    }
-}
-
 function getFilteredVolumes(vmStoragePool, disks) {
     const usedDiskPaths = Object.getOwnPropertyNames(disks)
             .filter(target => disks[target].source && (disks[target].source.file || disks[target].source.volume))
@@ -148,7 +138,7 @@ const PoolRow = ({ idPrefix, onValueChanged, storagePoolName, vmStoragePools })
     );
 };
 
-class PerformanceOptions extends React.Component {
+class AdditionalOptions extends React.Component {
     constructor(props) {
         super(props);
         this.state = { expanded: false };
@@ -156,14 +146,15 @@ class PerformanceOptions extends React.Component {
 
     render() {
         const cacheModes = ['default', 'none', 'writethrough', 'writeback', 'directsync', 'unsafe'];
+        const busTypes = ['sata', 'scsi', 'usb', 'virtio'];
 
         return (
             <>
                 <div className='expand-collapse-pf' id='expand-collapse-button'>
                     <div className='expand-collapse-pf-link-container'>
-                        <button className='btn btn-link' onClick={() => this.setState({ expanded: !this.state.expanded })}>
+                        <button className='pf-c-button pf-m-inline pf-m-link' onClick={() => this.setState({ expanded: !this.state.expanded })}>
                             { this.state.expanded ? <span className='fa fa-angle-down' /> : <span className='fa fa-angle-right' /> }
-                            { this.state.expanded ? _("Hide Performance Options") : _("Show Performance Options")}
+                            { this.state.expanded ? _("Hide Additional Options") : _("Show Additional Options")}
                         </button>
                         <span className="expand-collapse-pf-separator bordered" />
                     </div>
@@ -185,6 +176,22 @@ class PerformanceOptions extends React.Component {
                             );
                         })}
                     </Select.Select>
+
+                    <label className='control-label' htmlFor='bus-type'>
+                        {_("Bus")}
+                    </label>
+                    <Select.Select id='bus-type'
+                        onChange={value => this.props.onValueChanged('busType', value)}
+                        initial={this.props.busType}
+                        extraClass='form-control ct-form-split'>
+                        {busTypes.map(busType => {
+                            return (
+                                <Select.SelectEntry data={busType} key={busType}>
+                                    {busType}
+                                </Select.SelectEntry>
+                            );
+                        })}
+                    </Select.Select>
                 </>}
             </>
         );
@@ -209,20 +216,32 @@ const CreateNewDisk = ({ idPrefix, onValueChanged, dialogValues, vmStoragePools,
                                   storagePool={storagePool}
                                   dialogValues={dialogValues}
                                   onValueChanged={onValueChanged} />
-                <hr />
-                <PermanentChange idPrefix={idPrefix}
-                                 permanent={dialogValues.permanent}
-                                 onValueChanged={onValueChanged}
-                                 provider={provider}
-                                 vm={vm} />
-                {provider.name == 'LibvirtDBus' && <PerformanceOptions cacheMode={dialogValues.cacheMode}
-                                    onValueChanged={onValueChanged} />}
             </>}
         </>
     );
 };
 
-const UseExistingDisk = ({ idPrefix, onValueChanged, dialogValues, vmStoragePools, provider, vm }) => {
+const ChangeShareable = ({ idPrefix, vms, storagePool, volumeName, onValueChanged, providerName }) => {
+    const isVolumeUsed = getStorageVolumesUsage(vms, storagePool);
+    const volume = storagePool.volumes.find(vol => vol.name === volumeName);
+
+    if (!isVolumeUsed[volumeName] || (isVolumeUsed[volumeName].length === 0))
+        return null;
+
+    const vmsUsing = isVolumeUsed[volumeName].join(', ') + '.';
+    let text = _("This volume is already used by: ") + vmsUsing;
+    if (providerName == 'LibvirtDBus' && volume.format === "raw")
+        text += _(" Attaching it will make this disk shareable for every VM using it.");
+
+    return (<>
+        <span id={`${idPrefix}-vms-usage`} className='idle-message'>
+            <i className='pficon pficon-warning-triangle-o' />
+            <span>{text}</span>
+        </span>
+    </>);
+};
+
+const UseExistingDisk = ({ idPrefix, onValueChanged, dialogValues, vmStoragePools, provider, vm, vms }) => {
     return (
         <>
             <hr />
@@ -239,59 +258,18 @@ const UseExistingDisk = ({ idPrefix, onValueChanged, dialogValues, vmStoragePool
                                       onValueChanged={onValueChanged}
                                       vmStoragePools={vmStoragePools}
                                       vmDisks={vm.disks} />
-                <hr />
-                <PermanentChange idPrefix={idPrefix}
-                                 permanent={dialogValues.PermanentChange}
-                                 onValueChanged={onValueChanged}
-                                 provider={provider}
-                                 vm={vm} />
-                {provider.name == 'LibvirtDBus' && <PerformanceOptions cacheMode={dialogValues.cacheMode}
-                                    onValueChanged={onValueChanged} />}
+                <ChangeShareable idPrefix={idPrefix}
+                    vms={vms}
+                    storagePool={vmStoragePools.find(pool => pool.name === dialogValues.storagePoolName)}
+                    volumeName={dialogValues.existingVolumeName}
+                    onValueChanged={onValueChanged}
+                    providerName={provider.name} />
             </>}
         </>
     );
 };
 
-export class AddDiskAction extends React.Component {
-    constructor(props) {
-        super(props);
-        this.state = { showModal: false };
-        this.open = this.open.bind(this);
-        this.close = this.close.bind(this);
-    }
-
-    close() {
-        this.setState({ showModal: false });
-    }
-
-    open() {
-        // Refresh storage volume list before displaying the dialog.
-        // There are recently no Libvirt events for storage volumes and polling is ugly.
-        // https://bugzilla.redhat.com/show_bug.cgi?id=1578836
-        this.props.dispatch(getAllStoragePools(this.props.vm.connectionName))
-                .then(() => {
-                    this.setState({ showModal: true });
-                });
-    }
-
-    render() {
-        const { vm, storagePools, provider, dispatch } = this.props;
-        const idPrefix = `${this.props.idPrefix}-adddisk`;
-        const filteredStoragePools = storagePools
-                .filter(pool => pool.active);
-
-        return (
-            <div id={`${idPrefix}-add-dialog-full`}>
-                <Button id={`${idPrefix}`} bsStyle='primary' onClick={this.open} className='pull-right'>
-                    {_("Add Disk")}
-                </Button>
-                { this.state.showModal && <AddDiskModalBody close={this.close} dispatch={dispatch} idPrefix={this.props.idPrefix} vm={vm} storagePools={filteredStoragePools} provider={provider} /> }
-            </div>
-        );
-    }
-}
-
-class AddDiskModalBody extends React.Component {
+export class AddDiskModalBody extends React.Component {
     constructor(props) {
         super(props);
         this.state = this.initialState;
@@ -303,7 +281,9 @@ class AddDiskModalBody extends React.Component {
 
     get initialState() {
         const { vm, storagePools, provider } = this.props;
-        const availableTarget = getNextAvailableTarget(vm);
+        const defaultBus = 'virtio';
+        const existingTargets = Object.getOwnPropertyNames(vm.disks);
+        const availableTarget = getNextAvailableTarget(existingTargets, defaultBus);
         const sortFunction = (poolA, poolB) => poolA.name.localeCompare(poolB.name);
         let defaultPool;
         if (storagePools.length > 0)
@@ -324,6 +304,8 @@ class AddDiskModalBody extends React.Component {
             hotplug: provider.isRunning(vm.state), // must be kept false for a down VM; the value is not being changed by user
             addDiskInProgress: false,
             cacheMode: 'default',
+            busType: defaultBus,
+            updateDisks: false,
         };
     }
 
@@ -336,10 +318,19 @@ class AddDiskModalBody extends React.Component {
 
     onValueChanged(key, value) {
         let stateDelta = {};
+        const { storagePools, vm } = this.props;
 
         switch (key) {
         case 'storagePoolName': {
+            const currentPool = storagePools.find(pool => pool.name === value && pool.connectionName === vm.connectionName);
+            const prevPool = storagePools.find(pool => pool.name === this.state.storagePoolName && pool.connectionName === vm.connectionName);
+
             this.setState({ storagePoolName: value });
+            // Reset the format only when the Format selection dropdown changes entries - otherwise just keep the old selection
+            // All pool types apart from 'disk' have either 'raw' or 'qcow2' format
+            if ((currentPool.type == 'disk' && prevPool.type != 'disk') || (currentPool.type != 'disk' && prevPool.type == 'disk'))
+                this.onValueChanged('format', getDefaultVolumeFormat(value));
+
             if (this.state.mode === USE_EXISTING) { // user changed pool
                 this.onValueChanged('existingVolumeName', this.getDefaultVolumeName(value));
             }
@@ -348,7 +339,6 @@ class AddDiskModalBody extends React.Component {
         case 'existingVolumeName': {
             stateDelta.existingVolumeName = value;
             this.setState(prevState => { // to prevent asynchronous for recursive call with existingVolumeName as a key
-                const { storagePools, vm } = this.props;
                 const pool = storagePools.find(pool => pool.name === prevState.storagePoolName && pool.connectionName === vm.connectionName);
                 stateDelta.format = getDefaultVolumeFormat(pool);
                 if (['dir', 'fs', 'netfs', 'gluster', 'vstorage'].indexOf(pool.type) > -1) {
@@ -361,14 +351,25 @@ class AddDiskModalBody extends React.Component {
             break;
         }
         case 'mode': {
-            stateDelta = this.initialState;
-            if (value === USE_EXISTING) { // user moved to USE_EXISTING subtab
-                stateDelta.mode = value;
-                const poolName = this.state.storagePoolName;
-                if (poolName)
-                    stateDelta.existingVolumeName = this.getDefaultVolumeName(poolName);
-            }
-            this.setState(stateDelta);
+            this.setState(prevState => { // to prevent asynchronous for recursive call with existingVolumeName as a key
+                stateDelta = this.initialState;
+                if (value === USE_EXISTING) { // user moved to USE_EXISTING subtab
+                    stateDelta.mode = value;
+                    const poolName = stateDelta.storagePoolName;
+                    if (poolName)
+                        this.onValueChanged('existingVolumeName', this.getDefaultVolumeName(poolName));
+                }
+
+                return stateDelta;
+            });
+
+            break;
+        }
+        case 'busType': {
+            const existingTargets = Object.getOwnPropertyNames(this.props.vm.disks);
+            const availableTarget = getNextAvailableTarget(existingTargets, value);
+            this.onValueChanged('target', availableTarget);
+            this.setState({ busType: value });
             break;
         }
         default:
@@ -381,7 +382,11 @@ class AddDiskModalBody extends React.Component {
     }
 
     onAddClicked() {
-        const { vm, dispatch } = this.props;
+        const { vm, dispatch, provider, close, vms, storagePools } = this.props;
+
+        // validate
+        if (!this.state.storagePoolName)
+            return this.dialogErrorSet(_("Please choose a storage pool"));
 
         if (this.state.mode === CREATE_NEW) {
             // validate
@@ -405,19 +410,27 @@ class AddDiskModalBody extends React.Component {
                 hotplug: this.state.hotplug,
                 vmName: vm.name,
                 vmId: vm.id,
-                cacheMode: this.state.cacheMode
+                cacheMode: this.state.cacheMode,
+                busType: this.state.busType
             }))
                     .fail(exc => {
                         this.setState({ addDiskInProgress: false });
                         this.dialogErrorSet(_("Disk failed to be created"), exc.message);
                     })
                     .then(() => { // force reload of VM data, events are not reliable (i.e. for a down VM)
-                        this.props.close();
+                        close();
                         return dispatch(getVm({ connectionName: vm.connectionName, name: vm.name, id: vm.id }));
                     });
         }
 
         // use existing volume
+        if (!this.state.existingVolumeName)
+            return this.dialogErrorSet(_("Please choose a volume"));
+
+        const storagePool = storagePools.find(pool => pool.name === this.state.storagePoolName);
+        const volume = storagePool.volumes.find(vol => vol.name === this.state.existingVolumeName);
+        const isVolumeUsed = getStorageVolumesUsage(vms, storagePool);
+
         return dispatch(attachDisk({
             connectionName: vm.connectionName,
             poolName: this.state.storagePoolName,
@@ -428,21 +441,40 @@ class AddDiskModalBody extends React.Component {
             hotplug: this.state.hotplug,
             vmName: vm.name,
             vmId: vm.id,
-            cacheMode: this.state.cacheMode
+            cacheMode: this.state.cacheMode,
+            shareable: provider.name == 'LibvirtDBus' && volume && volume.format === "raw" && isVolumeUsed[this.state.existingVolumeName],
+            busType: this.state.busType
         }))
                 .fail(exc => {
                     this.setState({ addDiskInProgress: false });
                     this.dialogErrorSet(_("Disk failed to be attached"), exc.message);
                 })
                 .then(() => { // force reload of VM data, events are not reliable (i.e. for a down VM)
-                    this.props.close();
+                    const promises = [];
+
+                    if (provider.name == 'LibvirtDBus' && volume.format === "raw" && isVolumeUsed[this.state.existingVolumeName]) {
+                        isVolumeUsed[this.state.existingVolumeName].forEach(vmName => {
+                            const vm = vms.find(vm => vm.name === vmName);
+                            const diskTarget = getStorageVolumeDiskTarget(vm, storagePool, this.state.existingVolumeName);
+
+                            promises.push(
+                                updateDiskAttributes({ connectionName: vm.connectionName, objPath: vm.id, readonly: false, shareable: true, target: diskTarget })
+                                        .fail(exc => this.dialogErrorSet(_("Disk settings could not be saved"), exc.message))
+                            );
+                        });
+
+                        Promise.all(promises)
+                                .then(() => close());
+                    } else {
+                        close();
+                    }
+
                     return dispatch(getVm({ connectionName: vm.connectionName, name: vm.name, id: vm.id }));
                 });
     }
 
     render() {
-        const { vm, storagePools, provider } = this.props;
-        const storagePoolsFiltered = storagePools.filter(pool => pool != undefined);
+        const { vm, storagePools, provider, vms } = this.props;
         const idPrefix = `${this.props.idPrefix}-adddisk`;
 
         const defaultBody = (
@@ -476,7 +508,7 @@ class AddDiskModalBody extends React.Component {
                     <CreateNewDisk idPrefix={`${idPrefix}-new`}
                                    onValueChanged={this.onValueChanged}
                                    dialogValues={this.state}
-                                   vmStoragePools={storagePoolsFiltered}
+                                   vmStoragePools={storagePools}
                                    provider={provider}
                                    vm={vm} />
                 )}
@@ -484,10 +516,22 @@ class AddDiskModalBody extends React.Component {
                     <UseExistingDisk idPrefix={`${idPrefix}-existing`}
                                      onValueChanged={this.onValueChanged}
                                      dialogValues={this.state}
-                                     vmStoragePools={storagePoolsFiltered}
+                                     vmStoragePools={storagePools}
                                      provider={provider}
+                                     vms={vms}
                                      vm={vm} />
                 )}
+                {vm.persistent && <>
+                    <hr />
+                    <PermanentChange idPrefix={idPrefix}
+                                     permanent={this.state.permanent}
+                                     onValueChanged={this.onValueChanged}
+                                     provider={provider}
+                                     vm={vm} />
+                </>}
+                {provider.name == 'LibvirtDBus' && <AdditionalOptions cacheMode={this.state.cacheMode}
+                                    onValueChanged={this.onValueChanged}
+                                    busType={this.state.busType} />}
             </div>
         );
 
@@ -495,7 +539,7 @@ class AddDiskModalBody extends React.Component {
             <Modal id={`${idPrefix}-dialog-modal-window`} show onHide={this.props.close}>
                 <Modal.Header>
                     <Modal.CloseButton onClick={this.props.close} />
-                    <Modal.Title> {`Add Disk`} </Modal.Title>
+                    <Modal.Title>{_("Add Disk")}</Modal.Title>
                 </Modal.Header>
                 <Modal.Body>
                     {defaultBody}
@@ -503,10 +547,10 @@ class AddDiskModalBody extends React.Component {
                 <Modal.Footer>
                     {this.state.dialogError && <ModalError dialogError={this.state.dialogError} dialogErrorDetail={this.state.dialogErrorDetail} />}
                     {this.state.addDiskInProgress && <div className="spinner spinner-sm pull-left" />}
-                    <Button id={`${idPrefix}-dialog-cancel`} bsStyle='default' className='btn-cancel' onClick={this.props.close}>
+                    <Button id={`${idPrefix}-dialog-cancel`} variant='secondary' className='btn-cancel' onClick={this.props.close}>
                         {_("Cancel")}
                     </Button>
-                    <Button id={`${idPrefix}-dialog-add`} bsStyle='primary' disabled={this.state.addDiskInProgress || storagePools.length == 0} onClick={this.onAddClicked}>
+                    <Button id={`${idPrefix}-dialog-add`} variant='primary' isDisabled={this.state.addDiskInProgress || storagePools.length == 0} onClick={this.onAddClicked}>
                         {_("Add")}
                     </Button>
                 </Modal.Footer>
