@@ -14,7 +14,6 @@ import {
 } from './actions/store-actions.js';
 
 import {
-    checkLibvirtStatus,
     getApiData,
     getHypervisorMaxVCPU,
     getLoggedInUser,
@@ -23,6 +22,7 @@ import {
 
 import {
     convertToUnit,
+    getNextAvailableTarget,
     logDebug,
     fileDownload,
     rephraseUI,
@@ -32,6 +32,9 @@ import {
 import {
     prepareDisksParam,
     prepareDisplaysParam,
+    prepareNICParam,
+    prepareVcpuParam,
+    prepareMemoryParam,
 } from './libvirtUtils.js';
 
 import {
@@ -118,6 +121,41 @@ export function getDomainElem(domXml) {
     return xmlDoc.getElementsByTagName("domain")[0];
 }
 
+export function getIfaceElemByMac(domxml, mac) {
+    const domainElem = getDomainElem(domxml);
+
+    if (!domainElem) {
+        console.warn(`Can't parse dumpxml, input: "${domainElem}"`);
+        return;
+    }
+
+    const devicesElem = domainElem.getElementsByTagName('devices')[0];
+    const ifaceElems = devicesElem.getElementsByTagName('interface');
+
+    if (ifaceElems) {
+        for (let i = 0; i < ifaceElems.length; i++) {
+            const ifaceElem = ifaceElems[i];
+            const macElem = ifaceElem.getElementsByTagName('mac')[0];
+            const address = macElem.getAttribute('address'); // identifier of the iface
+            if (address === mac) {
+                return new XMLSerializer().serializeToString(ifaceElem);
+            }
+        }
+    }
+}
+
+function getIfaceElem(netXml) {
+    const parser = new DOMParser();
+    const xmlDoc = parser.parseFromString(netXml, "application/xml");
+
+    if (!xmlDoc) {
+        console.warn(`Can't parse dumpxml, input: "${netXml}"`);
+        return;
+    }
+
+    return xmlDoc.getElementsByTagName("interface")[0];
+}
+
 function getNetworkElem(netXml) {
     const parser = new DOMParser();
     const xmlDoc = parser.parseFromString(netXml, "application/xml");
@@ -192,6 +230,8 @@ export function parseDumpxml(dispatch, connectionName, domXml, id_overwrite) {
     const osBoot = parseDumpxmlForOsBoot(osBootElems);
     const arch = osTypeElem.getAttribute("arch");
     const emulatedMachine = osTypeElem.getAttribute("machine");
+    const firmware = osElem.getAttribute("firmware");
+    const loaderElem = getSingleOptionalElem(osElem, "loader");
 
     const currentMemoryUnit = currentMemoryElem.getAttribute("unit");
     const currentMemory = convertToUnit(currentMemoryElem.childNodes[0].nodeValue, currentMemoryUnit, units.KiB);
@@ -225,6 +265,8 @@ export function parseDumpxml(dispatch, connectionName, domXml, id_overwrite) {
         id,
         osType,
         osBoot,
+        firmware,
+        loader: loaderElem ? loaderElem.textContent : undefined,
         arch,
         currentMemory,
         memory,
@@ -373,6 +415,7 @@ export function parseDumpxmlForDisks(devicesElem) {
                     pool: sourceElem ? sourceElem.getAttribute('pool') : undefined,
                     volume: sourceElem ? sourceElem.getAttribute('volume') : undefined,
                     protocol: sourceElem ? sourceElem.getAttribute('protocol') : undefined,
+                    name: sourceElem ? sourceElem.getAttribute('name') : undefined,
                     host: {
                         name: sourceHostElem ? sourceHostElem.getAttribute('name') : undefined,
                         port: sourceHostElem ? sourceHostElem.getAttribute('port') : undefined,
@@ -582,7 +625,7 @@ export function parseDumpxmlForInterfaces(devicesElem) {
                 name: interfaceElem.getAttribute('name') ? interfaceElem.getAttribute('name') : undefined, // Name of interface
                 target: targetElem ? targetElem.getAttribute('dev') : undefined,
                 mac: macElem.getAttribute('address'), // MAC address
-                model: modelElem.getAttribute('type'), // Device model
+                model: modelElem ? modelElem.getAttribute('type') : undefined, // Device model
                 aliasName: aliasElem ? aliasElem.getAttribute('name') : undefined,
                 virtualportType: virtualportElem ? virtualportElem.getAttribute('type') : undefined,
                 driverName: driverElem ? driverElem.getAttribute('name') : undefined,
@@ -605,6 +648,7 @@ export function parseDumpxmlForInterfaces(devicesElem) {
                 address: {
                     bus: addressElem ? addressElem.getAttribute('bus') : undefined,
                     function: addressElem ? addressElem.getAttribute('function') : undefined,
+                    slot: addressElem ? addressElem.getAttribute('slot') : undefined,
                 },
             };
             interfaces.push(networkInterface);
@@ -622,6 +666,15 @@ export function parseDumpxmlMachinesMetadataElement(metadataElem, name) {
     return subElems.length > 0 ? subElems[0].textContent : null;
 }
 
+export function parseIfaceDumpxml(ifaceXml) {
+    const retObj = {};
+    const ifaceElem = getIfaceElem(ifaceXml);
+
+    retObj.type = ifaceElem.getAttribute("type");
+
+    return retObj;
+}
+
 export function parseNetDumpxml(netXml) {
     const retObj = {};
     const netElem = getNetworkElem(netXml);
@@ -846,29 +899,77 @@ export function parseStorageVolumeDumpxml(connectionName, storageVolumeXml, id_o
     };
 }
 
-export function resolveUiState(dispatch, name) {
+export function resolveUiState(dispatch, name, connectionName) {
     const result = {
         // used just the first time vm is shown
         initiallyExpanded: false,
         initiallyOpenedConsoleTab: false,
     };
 
-    const uiState = store.getState().ui.vms[name];
+    const uiState = store.getState().ui.vms.find(vm => vm.name == name && vm.connectionName == connectionName);
 
     if (uiState) {
         result.initiallyExpanded = uiState.expanded;
         result.initiallyOpenedConsoleTab = uiState.openConsoleTab;
 
         if (uiState.installInProgress) {
-            removeVmCreateInProgress(dispatch, name);
+            removeVmCreateInProgress(dispatch, name, connectionName);
         } else {
-            clearVmUiState(dispatch, name);
+            clearVmUiState(dispatch, name, connectionName);
         }
     }
 
     return result;
 }
 
+export function updateDisk({ domXml, diskTarget, readonly, shareable, busType, existingTargets }) {
+    const domainElem = getDomainElem(domXml);
+    if (!domainElem)
+        throw new Error("updateBootOrder: domXML has no domain element");
+
+    const deviceElem = domainElem.getElementsByTagName("devices")[0];
+    const disks = deviceElem.getElementsByTagName("disk");
+
+    for (let i = 0; i < disks.length; i++) {
+        const disk = disks[i];
+        const target = disk.getElementsByTagName("target")[0].getAttribute("dev");
+        if (target == diskTarget) {
+            let shareAbleElem = getSingleOptionalElem(disk, "shareable");
+            if (!shareAbleElem && shareable) {
+                shareAbleElem = document.createElement("shareable");
+                disk.appendChild(shareAbleElem);
+            } else if (shareAbleElem && !shareable) {
+                shareAbleElem.remove();
+            }
+
+            let readOnlyElem = getSingleOptionalElem(disk, "readonly");
+            if (!readOnlyElem && readonly) {
+                readOnlyElem = document.createElement("readonly");
+                disk.appendChild(readOnlyElem);
+            } else if (readOnlyElem && !readonly) {
+                readOnlyElem.remove();
+            }
+
+            const targetElem = disk.getElementsByTagName("target")[0];
+            const oldBusType = targetElem.getAttribute("bus");
+            if (busType && oldBusType !== busType) {
+                targetElem.setAttribute("bus", busType);
+                const newTarget = getNextAvailableTarget(existingTargets, busType);
+                targetElem.setAttribute("dev", newTarget);
+
+                const addressElem = getSingleOptionalElem(disk, "address");
+                addressElem.remove();
+            }
+        }
+    }
+
+    const tmp = document.createElement("div");
+
+    tmp.appendChild(domainElem);
+
+    return tmp.innerHTML;
+}
+
 export function unknownConnectionName(action, libvirtServiceName) {
     return dispatch => {
         return cockpit.user().done(loggedUser => {
@@ -878,11 +979,7 @@ export function unknownConnectionName(action, libvirtServiceName) {
                         // https://bugzilla.redhat.com/show_bug.cgi?id=1045069
                         connectionName => canLoggedUserConnectSession(connectionName, loggedUser))
                     .map(connectionName => dispatch(action(connectionName, libvirtServiceName)));
-
-            // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-            // https://github.com/cockpit-project/cockpit/issues/10956
-            // eslint-disable-next-line cockpit/no-cockpit-all
-            return cockpit.all(promises);
+            return Promise.all(promises);
         });
     };
 }
@@ -1191,14 +1288,14 @@ export function CONSOLE_VM({
     };
 }
 
-export function CREATE_VM({ connectionName, vmName, source, sourceType, os, memorySize, storageSize, startVm, storagePool, storageVolume }) {
+export function CREATE_VM({ connectionName, vmName, source, sourceType, os, memorySize, storageSize, startVm, storagePool, storageVolume, unattended, userPassword, rootPassword, profile }) {
     logDebug(`${this.name}.CREATE_VM(${vmName}):`);
     return dispatch => {
         // shows dummy vm  until we get vm from virsh (cleans up inProgress)
-        setVmCreateInProgress(dispatch, vmName, { openConsoleTab: startVm });
+        setVmCreateInProgress(dispatch, vmName, connectionName, { openConsoleTab: startVm });
 
         if (startVm) {
-            setVmInstallInProgress(dispatch, vmName);
+            setVmInstallInProgress(dispatch, vmName, connectionName);
         }
 
         return cockpit.script(createVmScript, [
@@ -1212,15 +1309,19 @@ export function CREATE_VM({ connectionName, vmName, source, sourceType, os, memo
             startVm,
             storagePool,
             storageVolume,
+            unattended,
+            userPassword,
+            rootPassword,
+            profile,
         ], { err: "message", environ: ['LC_ALL=C'] })
                 .done(() => {
-                    finishVmCreateInProgress(dispatch, vmName);
+                    finishVmCreateInProgress(dispatch, vmName, connectionName);
                     if (startVm) {
-                        clearVmUiState(dispatch, vmName);
+                        clearVmUiState(dispatch, vmName, connectionName);
                     }
                 })
                 .fail((exception, data) => {
-                    clearVmUiState(dispatch, vmName); // inProgress cleanup
+                    clearVmUiState(dispatch, vmName, connectionName); // inProgress cleanup
                     console.info(`spawn 'vm creation' returned error: "${JSON.stringify(exception)}", data: "${JSON.stringify(data)}"`);
                 });
     };
@@ -1249,7 +1350,7 @@ export function GET_LOGGED_IN_USER() {
 
 export function GET_OS_INFO_LIST () {
     logDebug(`${this.name}.GET_OS_INFO_LIST():`);
-    return dispatch => python.spawn(getOSListScript, null, { err: "message", environ: ['LC_ALL=en_US.UTF-8'] })
+    return dispatch => python.spawn(getOSListScript, null, { err: "message", environ: ['LC_ALL=C.UTF-8'] })
             .then(osList => {
                 parseOsInfoList(dispatch, osList);
             })
@@ -1264,7 +1365,7 @@ export function INIT_DATA_RETRIEVAL () {
     return dispatch => {
         dispatch(getOsInfoList());
         dispatch(getLoggedInUser());
-        return cockpit.script(getLibvirtServiceNameScript, null, { err: "message", environ: ['LC_ALL=en_US.UTF-8'] })
+        return cockpit.script(getLibvirtServiceNameScript, null, { err: "message", environ: ['LC_ALL=C.UTF-8'] })
                 .then(serviceName => {
                     const match = serviceName.match(/([^\s]+)/);
                     const name = match ? match[0] : null;
@@ -1283,12 +1384,12 @@ export function INIT_DATA_RETRIEVAL () {
     };
 }
 
-export function INSTALL_VM({ name, vcpus, currentMemory, metadata, disks, displays, connectionName, onAddErrorNotification }) {
+export function INSTALL_VM({ name, vcpus, cpu, currentMemory, memory, metadata, disks, displays, interfaces, firmware, autostart, connectionName, onAddErrorNotification }) {
     logDebug(`${this.name}.INSTALL_VM(${name}):`);
     return dispatch => {
         // shows dummy vm until we get vm from virsh (cleans up inProgress)
         // vm should be returned even if script fails
-        setVmInstallInProgress(dispatch, name);
+        setVmInstallInProgress(dispatch, name, connectionName);
 
         return cockpit.script(installVmScript, [
             connectionName,
@@ -1296,14 +1397,17 @@ export function INSTALL_VM({ name, vcpus, currentMemory, metadata, disks, displa
             metadata.installSourceType,
             metadata.installSource,
             metadata.osVariant,
-            convertToUnit(currentMemory, units.KiB, units.MiB),
-            vcpus.count,
+            prepareMemoryParam(convertToUnit(currentMemory, units.KiB, units.MiB), convertToUnit(memory, units.KiB, units.MiB)),
+            prepareVcpuParam(vcpus, cpu),
             prepareDisksParam(disks),
             prepareDisplaysParam(displays),
+            prepareNICParam(interfaces),
+            firmware == "efi" ? 'uefi' : '',
+            autostart,
         ], { err: "message", environ: ['LC_ALL=C'] })
-                .done(() => clearVmUiState(dispatch, name))
+                .done(() => clearVmUiState(dispatch, name, connectionName))
                 .fail(ex => {
-                    clearVmUiState(dispatch, name); // inProgress cleanup
+                    clearVmUiState(dispatch, name, connectionName); // inProgress cleanup
                     buildScriptTimeoutFailHandler(
                         () => onAddErrorNotification({ text: cockpit.format(_("VM $0 failed to get installed"), name), detail: ex.message })
                         , VMS_CONFIG.WaitForRetryInstallVm);
@@ -1315,9 +1419,6 @@ export function START_LIBVIRT({ serviceName }) {
     logDebug(`${this.name}.START_LIBVIRT`);
     return dispatch => {
         return service.proxy(serviceName).start()
-                .done(() => {
-                    dispatch(checkLibvirtStatus(serviceName));
-                })
                 .fail(exception => {
                     console.info(`starting libvirt failed: "${JSON.stringify(exception)}"`);
                 });
