@@ -48,12 +48,32 @@ utils.list_interfaces().then(interfaces => {
 const firewalld_service = service.proxy('firewalld');
 var firewalld_dbus = null;
 
+firewall.debouncedGetZones = debounce(300, () => {
+    getZones()
+            .then(() => getServices())
+            .then(() => firewall.debouncedEvent('changed'))
+            .catch(error => console.warn(error));
+});
+
+/* As certain dbus signal callbacks might change the firewall frequently
+ * in a short period of time, prevent rapid succession of renders by
+ * debouncing the ('changed') event */
+firewall.debouncedEvent = debounce(300, event => firewall.dispatchEvent(event));
+
+/* As a service might be removed from multiple zones at the same time,
+ * prevent rapid succession of GetServices call */
+firewall.debouncedGetServices = debounce(300, () => {
+    getServices().then(() => firewall.debouncedEvent('changed'));
+});
+
 function initFirewalldDbus() {
     firewalld_dbus = cockpit.dbus('org.fedoraproject.FirewallD1', { superuser: "try" });
 
     firewalld_dbus.addEventListener('owner', (event, owner) => {
         firewall.enabled = !!owner;
 
+        firewall.zones = {};
+        firewall.activeZones = new Set();
         firewall.services = {};
         firewall.enabledServices = new Set();
 
@@ -62,17 +82,9 @@ function initFirewalldDbus() {
             return;
         }
 
-        /* As certain dbus signal callbacks might change the firewall frequently
-         * in a short period of time, prevent rapid succession of renders by
-         * debouncing the ('changed') event */
-        firewall.debouncedEvent = debounce(300, event => firewall.dispatchEvent(event));
-
-        /* As a service might be removed from multiple zones at the same time,
-         * prevent rapid succession of GetServices call */
-        firewall.debouncedGetServices = debounce(300, getServices);
-
         getZones()
                 .then(() => getServices())
+                .then(() => firewall.debouncedEvent('changed'))
                 .catch(error => console.warn(error));
     });
 
@@ -107,9 +119,42 @@ function initFirewalldDbus() {
         interface: 'org.fedoraproject.FirewallD1',
         path: '/org/fedoraproject/FirewallD1',
         member: 'Reloaded'
-    }, () => getZones().then(() => getServices()));
+    }, () => firewall.debouncedGetZones());
 
-    getDefaultZonePath();
+    /* There are two APIs available, changeZoneOf(Interface|Source) and
+     * add(Interface|Source). Listen to both of them for any background changes
+     * to zones. */
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'ZoneOfInterfaceChanged'
+    }, () => firewall.debouncedGetZones());
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'ZoneOfSourceChanged'
+    }, () => firewall.debouncedGetZones());
+
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'InterfaceAdded'
+    }, () => firewall.debouncedGetZones());
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'SourceAdded'
+    }, () => firewall.debouncedGetZones());
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'InterfaceRemoved'
+    }, () => firewall.debouncedGetZones());
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'SourceRemoved'
+    }, () => firewall.debouncedGetZones());
 }
 
 firewalld_service.addEventListener('changed', () => {
@@ -132,7 +177,9 @@ function getZones() {
                                'org.fedoraproject.FirewallD1.zone',
                                'getActiveZones', [])
             .then(reply => fetchZoneInfos(Object.keys(reply[0])))
-            .then(zones => zones.map(z => firewall.activeZones.add(z.id)))
+            .then(zones => {
+                firewall.activeZones = new Set(zones.map(z => z.id));
+            })
             .then(() => firewalld_dbus.call('/org/fedoraproject/FirewallD1',
                                             'org.fedoraproject.FirewallD1',
                                             'getDefaultZone', []))
@@ -146,11 +193,10 @@ function getZones() {
 }
 
 function getServices() {
+    if (firewall.readonly)
+        return Promise.resolve();
     firewall.enabledServices = new Set();
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    return cockpit.all([...firewall.activeZones].map(z => {
+    return Promise.all([...firewall.activeZones].map(z => {
         return firewalld_dbus.call('/org/fedoraproject/FirewallD1',
                                    'org.fedoraproject.FirewallD1.zone',
                                    'getServices', [z])
@@ -162,19 +208,13 @@ function getServices() {
                         if (s.includes.length)
                             promises.push(fetchServiceInfos(s.includes));
                     }
-                    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-                    // https://github.com/cockpit-project/cockpit/issues/10956
-                    // eslint-disable-next-line cockpit/no-cockpit-all
-                    return cockpit.all(promises);
+                    return Promise.all(promises);
                 });
-    })).then(() => firewall.debouncedEvent('changed'));
+    }));
 }
 
 function fetchServiceInfos(services) {
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    var promises = cockpit.all(services.map(service => {
+    return Promise.all(services.map(service => {
         if (firewall.services[service])
             return firewall.services[service];
 
@@ -215,28 +255,24 @@ function fetchServiceInfos(services) {
                     Promise.reject(error);
                 });
     }));
-
-    /*
-     * Work around `cockpit.all()` returning results in individual arguments -
-     * that's just confusing and doesn't work with ES6 style functions.
-     */
-    return promises.then(function () {
-        /* FetchServiceInfos was called with on an empty array if arguments is
-         * [[]]. To prevent an array with 'undefined' as only element from
-         * returning, explicitly return an empty array.
-         */
-        const result = Array.prototype.slice.call(arguments);
-        if (result.length === 1 && result[0].length === 0)
-            return [];
-        return result;
-    });
 }
 
 function fetchZoneInfos(zones) {
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    const promises = cockpit.all(zones.map(zone => {
+    return Promise.all(zones.map(zone => {
+        if (firewall.readonly) {
+            const info = {
+                id: zone,
+                name: zone,
+                description: null,
+                target: null,
+                services: [],
+                ports: [],
+                interfaces: [],
+                source: [],
+            };
+            firewall.zones[zone] = info;
+            return info;
+        }
         return firewalld_dbus.call('/org/fedoraproject/FirewallD1',
                                    'org.fedoraproject.FirewallD1',
                                    'getZoneSettings', [zone])
@@ -256,11 +292,6 @@ function fetchZoneInfos(zones) {
                     return info;
                 });
     }));
-    return promises.then(function (zoneInfos) {
-        if (Array.isArray(zoneInfos) && zoneInfos.length === 0)
-            return [];
-        return Array.prototype.slice.call(arguments);
-    });
 }
 
 initFirewalldDbus();
@@ -268,7 +299,7 @@ initFirewalldDbus();
 cockpit.spawn(['sh', '-c', 'pkcheck --action-id org.fedoraproject.FirewallD1.all --process $$ --allow-user-interaction 2>&1'])
         .done(() => {
             firewall.readonly = false;
-            firewall.dispatchEvent('changed');
+            firewall.debouncedGetZones();
         });
 
 firewall.enable = () => Promise.all([firewalld_service.enable(), firewalld_service.start()]);
@@ -294,17 +325,6 @@ firewall.reload = () => {
             .catch(error => console.warn(error));
 };
 
-function getDefaultZonePath() {
-    return firewalld_dbus.call('/org/fedoraproject/FirewallD1',
-                               'org.fedoraproject.FirewallD1',
-                               'getDefaultZone', [])
-            .then(reply => firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
-                                               'org.fedoraproject.FirewallD1.config',
-                                               'getZoneByName', [reply[0]]))
-            .then(reply => reply[0])
-            .catch(error => console.warn(error));
-}
-
 /*
  * Remove a service from the specified zone (i.e., close its ports).
  *
@@ -365,31 +385,11 @@ firewall.addService = (zone, service) => {
  *
  * Returns a promise that resolves when all services are added.
  */
-firewall.addServices = (zones, services) => {
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    return cockpit.all(zones.map(z => services.map(s => firewall.addService(z, s))))
-            .then(function() {
-                const result = Array.prototype.slice.call(arguments);
-                if (result.length === 1 && result[0].length === 0)
-                    return [];
-                return result;
-            });
-};
+firewall.addServices = (zone, services) =>
+    Promise.all(services.map(s => firewall.addService(zone, s)));
 
-firewall.removeServiceFromZones = (zones, service) => {
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    return cockpit.all(zones.map(z => firewall.removeService(z, service)))
-            .then(function() {
-                const result = Array.prototype.slice.call(arguments);
-                if (result.length === 1 && result[0].length === 0)
-                    return [];
-                return result;
-            });
-};
+firewall.removeServiceFromZones = (zones, service) =>
+    Promise.all(zones.map(z => firewall.removeService(z, service)));
 
 firewall.activateZone = (zone, interfaces, sources) => {
     let promises = interfaces.map(i => firewalld_dbus.call('/org/fedoraproject/FirewallD1',
@@ -399,10 +399,7 @@ firewall.activateZone = (zone, interfaces, sources) => {
     promises = promises.concat(sources.map(s => firewalld_dbus.call('/org/fedoraproject/FirewallD1',
                                                                     'org.fedoraproject.FirewallD1.zone',
                                                                     'addSource', [zone, s])));
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    let p = cockpit.all(promises).then(() => firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
+    let p = Promise.all(promises).then(() => firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
                                                                  'org.fedoraproject.FirewallD1.config',
                                                                  'getZoneByName', [zone]));
     p = p.then(path => {
@@ -442,10 +439,7 @@ firewall.deactiveateZone = (zone) => {
     promises = promises.concat(zoneObject.source.map(s => firewalld_dbus.call('/org/fedoraproject/FirewallD1',
                                                                               'org.fedoraproject.FirewallD1.zone',
                                                                               'removeSource', [zone, s])));
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    let p = cockpit.all(promises).then(() => firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
+    let p = Promise.all(promises).then(() => firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
                                                                  'org.fedoraproject.FirewallD1.config',
                                                                  'getZoneByName', [zone]));
     p = p.then(path => {
