@@ -21,36 +21,27 @@ import cockpit from "cockpit";
 import React from "react";
 import ReactDOM from "react-dom";
 import {
-    Alert,
-    Button,
     ListView,
     Modal,
     OverlayTrigger,
     Tooltip
 } from "patternfly-react";
+import { Alert, Button } from '@patternfly/react-core';
+import { ExclamationCircleIcon, TrashIcon } from '@patternfly/react-icons';
 
 import firewall from "./firewall-client.js";
 import { Listing, ListingRow } from "cockpit-components-listing.jsx";
 import { OnOffSwitch } from "cockpit-components-onoff.jsx";
 import { ModalError } from "cockpit-components-inline-notification.jsx";
+import { EmptyStatePanel } from "cockpit-components-empty-state.jsx";
 
-import "page.css";
+import "page.scss";
 import "table.css";
-import "form-layout.less";
+import "form-layout.scss";
 import "./networking.css";
 
 const _ = cockpit.gettext;
 
-function EmptyState(props) {
-    return (
-        <div className="curtains-ct blank-slate-pf">
-            {props.icon && <div className={"blank-slate-pf-icon " + props.icon} />}
-            <h1>{props.title}</h1>
-            {props.children}
-        </div>
-    );
-}
-
 function ServiceRow(props) {
     var tcp = props.service.ports.filter(p => p.protocol.toUpperCase() == 'TCP');
     var udp = props.service.ports.filter(p => p.protocol.toUpperCase() == 'UDP');
@@ -75,29 +66,27 @@ function ServiceRow(props) {
         deleteButton = (
             <OverlayTrigger className="pull-right" placement="top"
                             overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                <button className="btn btn-danger pficon pficon-delete" disabled />
+                <span>
+                    <Button key={props.service.id + "-delete-button"} variant="danger" aria-label={cockpit.format(_("Not authorized to remove service $0"), props.service.id)} style={{ pointerEvents: 'none' }} isDisabled><TrashIcon /></Button>
+                </span>
             </OverlayTrigger>
         );
     } else {
-        deleteButton = <button className="btn btn-danger pficon pficon-delete" onClick={onRemoveService} />;
+        deleteButton = <Button key={props.service.id + "-delete-button"} variant="danger" onClick={onRemoveService} aria-label={cockpit.format(_("Remove service $0"), props.service.id)}><TrashIcon /></Button>;
     }
 
     var columns = [
         { name: props.service.name, header: true },
-        <div>
+        <div key={props.service.id + "tcp"}>
             { tcp.map(p => p.port).join(', ') }
         </div>,
-        <div>
+        <div key={props.service.id + "udp"}>
             { udp.map(p => p.port).join(', ') }
         </div>,
-        <div>
-            { props.zones.filter(z => z.services.indexOf(props.service.id) !== -1).map(z => z.name || z.id)
-                    .join(', ') }
-        </div>,
-        deleteButton
+        null
     ];
 
-    let description, includes, simpleBody;
+    let description, includes;
     if (props.service.description)
         description = <p>{props.service.description}</p>;
 
@@ -110,16 +99,16 @@ function ServiceRow(props) {
                     return <li key={service.id}><strong>{service.name}</strong>: {service.description}</li>;
             })} </ul></>;
     }
-    if (description || includes)
-        simpleBody = <>{description}{includes}</>;
+    const simpleBody = <>{description}{includes}</>;
 
     return <ListingRow key={props.service.id}
                        rowId={props.service.id}
                        columns={columns}
-                       simpleBody={simpleBody} />;
+                       simpleBody={simpleBody}
+                       listingActions={[deleteButton]} />;
 }
 
-function ZoneRow(props) {
+function ZoneSection(props) {
     function onRemoveZone(event) {
         if (event.button !== 0)
             return;
@@ -131,25 +120,56 @@ function ZoneRow(props) {
     let deleteButton;
     if (props.readonly) {
         deleteButton = (
-            <OverlayTrigger className="pull-right" placement="top"
+            <OverlayTrigger placement="top"
                             overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                <button className="btn btn-danger pficon pficon-delete" disabled />
+                <Button variant="danger" aria-label={cockpit.format(_("Not authorized to remove zone $0"), props.zone.id)} isDisabled><span className="pficon pficon-delete" /></Button>
             </OverlayTrigger>
         );
     } else {
-        deleteButton = <button className="btn btn-danger pficon pficon-delete" onClick={onRemoveZone} />;
+        deleteButton = <Button variant="danger" onClick={onRemoveZone} aria-label={cockpit.format(_("Remove zone $0"), props.zone.id)}><span className="pficon pficon-delete" /></Button>;
     }
 
-    const columns = [
-        { name: props.zone.name, header: true },
-        <>{ props.zone.id === firewall.defaultZone ? <><span className="fa fa-check" /> default</> : '' }</>,
-        <>{ props.zone.interfaces.length > 0 ? props.zone.interfaces.join(', ') : '*' }</>,
-        <>{ props.zone.source.length > 0 ? props.zone.source.join(', ') : '*' }</>,
-        deleteButton,
-    ];
-    return <ListingRow key={props.zone.id}
-                       rowId={props.zone.id}
-                       columns={columns} />;
+    let addServiceAction;
+    if (firewall.readonly) {
+        addServiceAction = (
+            <OverlayTrigger placement="top"
+                                overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
+                <Button variant="primary" className="pull-right add-services-button" aria-label={cockpit.format(_("Not authorized to add services to zone $0"), props.zone.id)} isDisabled> {_("Add Services")} </Button>
+            </OverlayTrigger>
+        );
+    } else {
+        addServiceAction = (
+            <Button variant="primary" onClick={() => props.openServicesDialog(props.zone.id, props.zone.id)} className="add-services-button" aria-label={cockpit.format(_("Add services to zone $0"), props.zone.id)}>
+                {_("Add Services")}
+            </Button>
+        );
+    }
+
+    return <div className="zone-section" data-id={props.zone.id}>
+        <div className="zone-section-heading">
+            <span>
+                <h4>{ cockpit.format(_("$0 zone"), props.zone.id) }</h4>
+                <div className="zone-section-targets">
+                    { props.zone.interfaces.length > 0 && <span className="zone-section-target"><strong>{_("Interfaces")}</strong> {props.zone.interfaces.join(", ")}</span> }
+                    { props.zone.source.length > 0 && <span className="zone-section-target"><strong>{_("Addresses")}</strong> {props.zone.source.join(", ")}</span> }
+                </div>
+            </span>
+            <div className="zone-section-buttons">{deleteButton}{addServiceAction}</div>
+        </div>
+        {props.zone.services.length > 0 &&
+        <Listing columnTitles={[_("Service"), _("TCP"), _("UDP"), ""]}
+                     emptyCaption={_("There are no active services in this zone")}>
+            { props.zone.services.map(s => {
+                if (s in firewall.services)
+                    return <ServiceRow key={firewall.services[s].id}
+                                       service={firewall.services[s]}
+                                       onRemoveService={service => props.onRemoveService(props.zone.id, service)}
+                                       readonly={firewall.readonly} />;
+            })
+            }
+        </Listing>
+        }
+    </div>;
 }
 
 class SearchInput extends React.Component {
@@ -224,8 +244,6 @@ class AddServicesModal extends React.Component {
             custom_udp_ports: [],
             custom_tcp_value: "",
             custom_udp_value: "",
-            /* If only one zone is active, automatically add services to that zone */
-            zones: firewall.activeZones.size === 1 ? [firewall.defaultZone] : [],
             dialogError: null,
             dialogErrorDetail: null,
         };
@@ -239,7 +257,6 @@ class AddServicesModal extends React.Component {
         this.parseServices = this.parseServices.bind(this);
         this.generateName = this.generateName.bind(this);
         this.onToggleType = this.onToggleType.bind(this);
-        this.onToggleZone = this.onToggleZone.bind(this);
     }
 
     createPorts() {
@@ -252,9 +269,9 @@ class AddServicesModal extends React.Component {
     save() {
         let p;
         if (this.state.custom) {
-            p = firewall.createService(this.state.custom_id, this.state.custom_name, this.createPorts(), this.state.zones);
+            p = firewall.createService(this.state.custom_id, this.state.custom_name, this.createPorts(), this.props.zoneId);
         } else {
-            p = firewall.addServices(this.state.zones, [...this.state.selected]);
+            p = firewall.addServices(this.props.zoneId, [...this.state.selected]);
         }
         p.then(() => this.props.close())
                 .catch(error => {
@@ -430,15 +447,6 @@ class AddServicesModal extends React.Component {
         });
     }
 
-    onToggleZone(event) {
-        const zone = event.target.value;
-        this.setState(state => {
-            if (state.zones.indexOf(zone) === -1)
-                return { zones: state.zones.concat(zone) };
-            return { zones: state.zones.filter(z => z !== zone) };
-        });
-    }
-
     componentDidMount() {
         firewall.getAvailableServices()
                 .then(services => this.setState({
@@ -471,39 +479,19 @@ class AddServicesModal extends React.Component {
         else
             services = this.state.services;
 
-        // hide services which have been enabled in all zones
-        if (services) {
-            services = services.filter(s => {
-                let allZonesContainService = true;
-                for (const zone of firewall.activeZones)
-                    allZonesContainService &= firewall.zones[zone].services.indexOf(s.id) !== -1;
-                return !allZonesContainService;
-            });
-        }
+        // hide services which have been enabled in the zone
+        if (services)
+            services = services.filter(s => firewall.zones[this.props.zoneId].services.indexOf(s.id) === -1);
 
-        var addText = this.state.custom ? _("Add Ports") : _("Add Services");
-        var zonesText = this.state.custom ? _("Add ports to the following zones:") : _("Add services to following zones:");
+        const addText = this.state.custom ? _("Add Ports") : _("Add Services");
+        const titleText = this.state.custom ? cockpit.format(_("Add ports to $0 zone"), this.props.zoneName) : cockpit.format(_("Add services to $0 zone"), this.props.zoneName);
         return (
             <Modal id="add-services-dialog" show onHide={this.props.close}>
                 <Modal.Header>
-                    <Modal.Title> {addText} </Modal.Title>
+                    <Modal.Title> {titleText} </Modal.Title>
                 </Modal.Header>
                 <div id="cockpit_modal_dialog">
                     <Modal.Body id="add-services-dialog-body">
-                        { firewall.activeZones.size > 1 &&
-                            <>
-                                <form className="ct-form horizontal">
-                                    <label htmlFor="zone-input">{zonesText}</label>
-                                    <fieldset id="zone-input">
-                                        { Array.from(firewall.activeZones).sort((a, b) => a.localeCompare(b))
-                                                .map(z =>
-                                                    <label className="radio" key={z}>
-                                                        <input type="checkbox" value={z} onChange={this.onToggleZone} />{ firewall.zones[z].name || z }{ z === firewall.defaultZone && " " + _("(default)") }
-                                                    </label>) }
-                                    </fieldset>
-                                </form>
-                                <hr />
-                            </>}
                         <form action="" className="toggle-body ct-form">
                             <label className="radio ct-form-full">
                                 <input type="radio" name="type" value="services" onChange={this.onToggleType} defaultChecked />
@@ -533,8 +521,7 @@ class AddServicesModal extends React.Component {
                                                                                                 onChange={this.onToggleService} /> }
                                                                         stacked
                                                                         heading={ <label htmlFor={"firewall-service-" + s.id}>{s.name}</label> }
-                                                                        description={ renderPorts(s) }>
-                                                            </ListView.Item>
+                                                                        description={ renderPorts(s) } />
                                                         ))
                                                     }
                                                 </ListView>
@@ -584,14 +571,14 @@ class AddServicesModal extends React.Component {
                         this.state.dialogError && <ModalError dialogError={this.state.dialogError} dialogErrorDetail={this.state.dialogErrorDetail} />
                     }
                     { !this.state.custom ||
-                        <Alert type="warning">
-                            { _("Adding custom ports will reload firewalld. A reload will result in the loss of any runtime-only configuration!")}
-                        </Alert>
+                        <Alert variant="warning"
+                            isInline
+                            title={_("Adding custom ports will reload firewalld. A reload will result in the loss of any runtime-only configuration!")} />
                     }
-                    <Button bsStyle='default' className='btn-cancel' onClick={this.props.close}>
+                    <Button variant='secondary' className='btn-cancel' onClick={this.props.close}>
                         {_("Cancel")}
                     </Button>
-                    <Button bsStyle='primary' onClick={this.save}>
+                    <Button variant='primary' onClick={this.save} aria-label={titleText}>
                         {addText}
                     </Button>
                 </Modal.Footer>
@@ -600,74 +587,6 @@ class AddServicesModal extends React.Component {
     }
 }
 
-class RemoveServicesModal extends React.Component {
-    constructor(props) {
-        super(props);
-
-        this.zonesWithService = Array.from(firewall.activeZones)
-                .filter(z => firewall.zones[z].services.indexOf(this.props.service) !== -1);
-        this.state = {
-            zones: this.zonesWithService.length === 1 ? this.zonesWithService : [],
-            dialogError: undefined,
-            dialogErrorDetail: undefined,
-        };
-        this.save = this.save.bind(this);
-        this.onToggleZone = this.onToggleZone.bind(this);
-    }
-
-    save() {
-        firewall.removeServiceFromZones(this.state.zones, this.props.service)
-                .then(() => this.props.close())
-                .catch(error => {
-                    this.setState({
-                        dialogError: _("Failed to remove service"),
-                        dialogErrorDetail: error.name + ": " + error.message,
-                    });
-                });
-    }
-
-    onToggleZone(event) {
-        const zone = event.target.value;
-        this.setState(state => {
-            if (state.zones.indexOf(zone) === -1)
-                return { zones: state.zones.concat(zone) };
-            return { zones: state.zones.filter(z => z !== zone) };
-        });
-    }
-
-    render() {
-        return (
-            <Modal id="remove-services-dialog" show onHide={this.props.close}>
-                <Modal.Header>
-                    <Modal.Title>{ _("Remove service from zones") }</Modal.Title>
-                </Modal.Header>
-                <Modal.Body id="remove-services-dialog-body">
-                    <form className="ct-form horizontal">
-                        <fieldset id="zone-input">
-                            { this.zonesWithService.map(z =>
-                                <label className="radio" key={z}>
-                                    <input type="checkbox" value={z} onChange={this.onToggleZone} defaultChecked={ this.zonesWithService.length === 1 } />
-                                    { z }{ z === firewall.defaultZone && " " + _("(default)") }
-                                </label>) }
-                        </fieldset>
-                    </form>
-                </Modal.Body>
-                <Modal.Footer>
-                    {
-                        this.state.dialogError && <ModalError dialogError={this.state.dialogError} dialogErrorDetail={this.state.dialogErrorDetail} />
-                    }
-                    <Button bsStyle="default" className="btn-cancel" onClick={this.props.close}>
-                        { _("Cancel") }
-                    </Button>
-                    <Button bsStyle="primary" onClick={this.save} disabled={ this.zonesWithService.length === 0 || this.state.zones.length === 0 }>
-                        { _("Remove service") }
-                    </Button>
-                </Modal.Footer>
-            </Modal>
-        );
-    }
-}
-
 class ActivateZoneModal extends React.Component {
     constructor() {
         super();
@@ -703,15 +622,22 @@ class ActivateZoneModal extends React.Component {
     }
 
     save() {
+        let p;
+        if (firewall.zones[this.state.zone].services.indexOf("cockpit") === -1)
+            p = firewall.addService(this.state.zone, "cockpit");
+        else
+            p = Promise.resolve();
+
         const sources = this.state.ipRange === "ip-range" ? this.state.ipRangeValue.split(",").map(ip => ip.trim()) : [];
-        firewall.activateZone(this.state.zone, [...this.state.interfaces], sources)
-                .then(() => this.props.close())
-                .catch(error => {
-                    this.setState({
-                        dialogError: _("Failed to add zone"),
-                        dialogErrorDetail: error.name + ": " + error.message,
-                    });
-                });
+        p.then(() =>
+            firewall.activateZone(this.state.zone, [...this.state.interfaces], sources)
+                    .then(() => this.props.close())
+                    .catch(error => {
+                        this.setState({
+                            dialogError: _("Failed to add zone"),
+                            dialogErrorDetail: error.name + ": " + error.message,
+                        });
+                    }));
     }
 
     render() {
@@ -764,6 +690,7 @@ class ActivateZoneModal extends React.Component {
                         <label htmlFor="add-zone-services-readonly" className="control-label">{ _("Included services") }</label>
                         <div id="add-zone-services-readonly">
                             { (this.state.zone && firewall.zones[this.state.zone].services.join(", ")) || _("None") }
+                            <legend>{_("The cockpit service is automatically included")}</legend>
                         </div>
 
                         <label htmlFor="add-zone-interface" className="control-label">{ _("Interfaces") }</label>
@@ -792,6 +719,7 @@ class ActivateZoneModal extends React.Component {
                             </label>
                             { this.state.ipRange === "ip-range" && <input id="add-zone-ip" onChange={e => this.onChange("ipRangeValue", e.target.value)} /> }
                         </div>
+                        <div>{ this.state.ipRange === "ip-range" && <legend>{_("IP address with routing prefix. Separate multiple values with a comma. Example: 192.0.2.0/24, 2001:db8::/32")}</legend> }</div>
                     </form>
                 </Modal.Body>
                 <Modal.Footer>
@@ -799,10 +727,10 @@ class ActivateZoneModal extends React.Component {
                         this.state.dialogError && <ModalError dialogError={this.state.dialogError} dialogErrorDetail={this.state.dialogErrorDetail} />
                     }
 
-                    <Button bsStyle="default" className="btn-cancel" onClick={this.props.close}>
+                    <Button variant="secondary" className="btn-cancel" onClick={this.props.close}>
                         { _("Cancel") }
                     </Button>
-                    <Button bsStyle="primary" onClick={this.save} disabled={this.state.zone === null ||
+                    <Button variant="primary" onClick={this.save} isDisabled={this.state.zone === null ||
                                                                             (this.state.interfaces.size === 0 && this.state.ipRange === "ip-entire-subnet") ||
                                                                             (this.state.ipRange === "ip-range" && !this.state.ipRangeValue)}>
                         { _("Add zone") }
@@ -813,22 +741,45 @@ class ActivateZoneModal extends React.Component {
     }
 }
 
+function DeleteConfirmationModal(props) {
+    return (
+        <Modal id="delete-confirmation-dialog" show>
+            <Modal.Header>
+                <Modal.Title>{ props.title }</Modal.Title>
+            </Modal.Header>
+            <Modal.Body className="delete-confirmation-body">
+                {props.body && <span className="fa fa-exclamation-triangle" />}
+                <div>{props.body}</div>
+            </Modal.Body>
+            <Modal.Footer>
+                <Button variant="secondary" className="btn-cancel" onClick={props.onCancel}>
+                    { _("Cancel") }
+                </Button>
+                <Button variant="danger" onClick={props.onDelete} aria-label={cockpit.format(_("Confirm removal of $0"), props.target)}>
+                    { _("Delete") }
+                </Button>
+            </Modal.Footer>
+        </Modal>
+    );
+}
+
 export class Firewall extends React.Component {
     constructor() {
         super();
 
         this.state = {
-            showAddServicesModal: false,
-            showRemoveServicesModal: false,
+            addServicesModal: undefined,
+            deleteConfirmationModal: undefined,
             firewall,
             pendingTarget: null /* `null` for not pending */
         };
 
         this.onFirewallChanged = this.onFirewallChanged.bind(this);
         this.onSwitchChanged = this.onSwitchChanged.bind(this);
-        this.onRemoveService = this.onRemoveService.bind(this);
         this.openServicesDialog = this.openServicesDialog.bind(this);
         this.openAddZoneDialog = this.openAddZoneDialog.bind(this);
+        this.onRemoveZone = this.onRemoveZone.bind(this);
+        this.onRemoveService = this.onRemoveService.bind(this);
         this.close = this.close.bind(this);
     }
 
@@ -850,12 +801,44 @@ export class Firewall extends React.Component {
             firewall.disable();
     }
 
-    onRemoveService(service) {
-        this.setState({ showRemoveServicesModal: service });
+    onRemoveZone(zone) {
+        let body;
+        if (firewall.zones[zone].services.indexOf("cockpit") !== -1)
+            body = _("This zone contains the cockpit service. Make sure that this zone does not apply to your current web console connection.");
+        else
+            body = _("Removing the zone will remove all services within it.");
+        this.setState({
+            deleteConfirmationModal: <DeleteConfirmationModal title={ cockpit.format(_("Remove zone $0"), zone) }
+            body={body}
+            target={zone}
+            onCancel={ () =>
+                this.setState({ deleteConfirmationModal: undefined })
+            }
+        onDelete={ () => {
+            firewall.deactiveateZone(zone);
+            this.setState({ deleteConfirmationModal: undefined });
+        }} />
+        });
     }
 
-    onRemoveZone(zone) {
-        firewall.deactiveateZone(zone);
+    onRemoveService(zone, service) {
+        if (service === 'cockpit') {
+            const body = _("Removing the cockpit service might result in the web console becoming unreachable. Make sure that this zone does not apply to your current web console connection.");
+            this.setState({
+                deleteConfirmationModal: <DeleteConfirmationModal title={ cockpit.format(_("Remove $0 service from $1 zone"), service, zone) }
+                body={body}
+                target={service}
+                onCancel={ () =>
+                    this.setState({ deleteConfirmationModal: undefined })
+                }
+                onDelete={ () => {
+                    firewall.removeService(zone, service);
+                    this.setState({ deleteConfirmationModal: undefined });
+                }} />
+            });
+        } else {
+            firewall.removeService(zone, service);
+        }
     }
 
     componentDidMount() {
@@ -868,14 +851,14 @@ export class Firewall extends React.Component {
 
     close() {
         this.setState({
-            showAddServicesModal: false,
+            addServicesModal: undefined,
             showRemoveServicesModal: false,
             showActivateZoneModal: false,
         });
     }
 
-    openServicesDialog() {
-        this.setState({ showAddServicesModal: true });
+    openServicesDialog(zoneId, zoneName) {
+        this.setState({ addServicesModal: <AddServicesModal zoneId={zoneId} zoneName={zoneName} close={this.close} /> });
     }
 
     openAddZoneDialog() {
@@ -891,35 +874,22 @@ export class Firewall extends React.Component {
         }
 
         if (!this.state.firewall.installed) {
-            return (
-                <EmptyState title={_("Firewall is not available")} icon="fa fa-exclamation-circle">
-                    <p>{cockpit.format(_("Please install the $0 package"), "firewalld")}</p>
-                </EmptyState>
-            );
+            return <EmptyStatePanel title={ _("Firewall is not available") }
+                                    paragraph={ cockpit.format(_("Please install the $0 package"), "firewalld") }
+                                    icon={ ExclamationCircleIcon } />;
         }
 
-        var addServiceAction, addZoneAction;
+        var addZoneAction;
         if (this.state.firewall.readonly) {
-            addServiceAction = (
-                <OverlayTrigger className="pull-right" placement="top"
-                                overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                    <Button bsStyle="primary" className="pull-right" disabled> {_("Add Services")} </Button>
-                </OverlayTrigger>
-            );
             addZoneAction = (
                 <OverlayTrigger className="pull-right" placement="top"
                                 overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                    <Button bsStyle="primary" className="pull-right" disabled> {_("Add Zone")} </Button>
+                    <Button variant="primary" className="pull-right" id="add-zone-button" aria-label={_("Not authorized to add a new zone")} isDisabled> {_("Add Zone")} </Button>
                 </OverlayTrigger>
             );
         } else {
-            addServiceAction = (
-                <Button bsStyle="primary" onClick={this.openServicesDialog} className="pull-right" id="add-services-button">
-                    {_("Add Services")}
-                </Button>
-            );
             addZoneAction = (
-                <Button bsStyle="primary" onClick={this.openAddZoneDialog} className="pull-right" id="add-zone-button">
+                <Button variant="primary" onClick={this.openAddZoneDialog} className="pull-right" id="add-zone-button" aria-label={_("Add a new zone")}>
                     {_("Add Zone")}
                 </Button>
             );
@@ -932,57 +902,60 @@ export class Firewall extends React.Component {
         });
         services.sort((a, b) => a.name.localeCompare(b.name));
 
-        var zones = [...this.state.firewall.activeZones].map(id => {
-            const zone = this.state.firewall.zones[id];
-            zone.name = zone.name || id;
-            return zone;
-        });
+        var zones = [...this.state.firewall.activeZones].sort((z1, z2) =>
+            z1 === firewall.defaultZone ? -1 : z2 === firewall.defaultZone ? 1 : 0
+        ).map(id => this.state.firewall.zones[id]);
 
         var enabled = this.state.pendingTarget !== null ? this.state.pendingTarget : this.state.firewall.enabled;
 
+        let firewallOnOff;
+        if (firewall.readonly) {
+            firewallOnOff = <OverlayTrigger className="pull-right" placement="top"
+                                            overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
+                <OnOffSwitch state={enabled}
+                             onChange={this.onSwitchChanged}
+                             aria-label={enabled ? _("Not authorized to disable the firewall") : _("Not authorized to enable the firewall")}
+                             disabled />
+            </OverlayTrigger>;
+        } else {
+            firewallOnOff = <OnOffSwitch state={enabled}
+                                         disabled={!!this.state.pendingTarget}
+                                         onChange={this.onSwitchChanged}
+                                         aria-label={enabled ? _("Disable the firewall") : _("Enable the firewall")} />;
+        }
+
         return (
-            <div className="container-fluid page-ct">
-                <ol className="breadcrumb">
-                    <li><a tabIndex="0" onClick={go_up}>{_("Networking")}</a></li>
-                    <li className="active">{_("Firewall")}</li>
-                </ol>
-                <h1>
-                    {_("Firewall")}
-                    <OnOffSwitch state={enabled}
-                                 disabled={!!this.state.pendingTarget}
-                                 onChange={this.onSwitchChanged} />
-                </h1>
-                <div id="zones-listing">
-                    { enabled && <Listing title={_("Active zones")}
-                             columnTitles={[_("Zone"), "", _("Interfaces"), _("IP Range"), ""]}
-                             emptyCaption={_("No active zones")}
-                             actions={addZoneAction}>
-                        {
-                            zones.map(z => <ZoneRow key={z.id}
-                                                    zone={z}
-                                                    readonly={this.state.firewall.readonly}
-                                                    onRemoveZone={this.onRemoveZone} />)
-                        }
-                    </Listing> }
+            <>
+                <div id="firewall-heading">
+                    <ol className="breadcrumb">
+                        <li><button role="link" className="link-button" onClick={go_up}>{_("Networking")}</button></li>
+                        <li className="active">{_("Firewall")}</li>
+                    </ol>
+                    <div id="firewall-heading-title">
+                        <span id="firewall-heading-title-group">
+                            <h1>{_("Firewall")}</h1>
+                            { firewallOnOff }
+                        </span>
+                        { enabled && <span className="btn-group">{addZoneAction}</span> }
+                    </div>
                 </div>
-                <div id="services-listing">
-                    { enabled && <Listing title={_("Allowed Services")}
-                             columnTitles={[_("Service"), _("TCP"), _("UDP"), _("Zones"), ""]}
-                             emptyCaption={_("No open ports")}
-                             actions={addServiceAction}>
+                <div id="zones-listing" className="container-fluid page-ct">
+                    { enabled && <>
                         {
-                            services.map(s => <ServiceRow key={s.id}
-                                                      service={s}
-                                                      zones={zones}
-                                                      readonly={this.state.firewall.readonly}
-                                                      onRemoveService={this.onRemoveService} />)
+                            zones.map(z => <ZoneSection key={z.id}
+                                                        zone={z}
+                                                        openServicesDialog={this.openServicesDialog}
+                                                        readonly={this.state.firewall.readonly}
+                                                        onRemoveZone={this.onRemoveZone}
+                                                        onRemoveService={this.onRemoveService} />
+                            )
                         }
-                    </Listing> }
+                    </> }
                 </div>
-                { this.state.showAddServicesModal && <AddServicesModal close={this.close} /> }
-                { this.state.showRemoveServicesModal && <RemoveServicesModal service={this.state.showRemoveServicesModal} close={this.close} /> }
+                { this.state.addServicesModal !== undefined && this.state.addServicesModal }
+                { this.state.deleteConfirmationModal !== undefined && this.state.deleteConfirmationModal }
                 { this.state.showActivateZoneModal && <ActivateZoneModal close={this.close} /> }
-            </div>
+            </>
         );
     }
 }
