@@ -19,20 +19,25 @@
 
 import cockpit from "cockpit";
 import '../lib/polyfills.js'; // once per application
-import React from "react";
+import React, { useState, useEffect } from "react";
 import ReactDOM from 'react-dom';
 
 import moment from "moment";
 import { OverlayTrigger, Tooltip } from "patternfly-react";
-import Markdown from "react-remarkable";
+import { Button } from '@patternfly/react-core';
+import { RebootingIcon, CheckIcon, ExclamationCircleIcon } from "@patternfly/react-icons";
+import { Remarkable } from "remarkable";
 import AutoUpdates from "./autoupdates.jsx";
 import { History, PackageList } from "./history.jsx";
+import { page_status } from "notifications";
+import { EmptyStatePanel } from "cockpit-components-empty-state.jsx";
 
 import * as PK from "packagekit.js";
 
-import "listing.less";
+import "listing.scss";
 
 const _ = cockpit.gettext;
+moment.locale(cockpit.language);
 
 // "available" heading is built dynamically
 const STATE_HEADINGS = {
@@ -106,32 +111,26 @@ function cleanupChangelogLine(text) {
     return text.trim();
 }
 
-class Expander extends React.Component {
-    constructor() {
-        super();
-        this.state = { expanded: false };
-    }
+function Expander({ title, onExpand, children }) {
+    const [expanded, setExpanded] = useState(false);
 
-    componentDidUpdate(prevProps, prevState) {
-        if (this.props.onExpand && !prevState.expanded && this.state.expanded)
-            this.props.onExpand();
-    }
+    useEffect(() => {
+        if (expanded && onExpand)
+            onExpand();
+    }, [expanded, onExpand]);
 
-    render() {
-        const title = <a href="#">{this.props.title}</a>;
-        const cls = "expander-caret fa " + (this.state.expanded ? "fa-angle-down" : "fa-angle-right");
-        return (
-            <>
-                <div className="expander-title">
-                    <hr />
-                    <span onClick={() => this.setState({ expanded: !this.state.expanded })}>
-                        <i className={cls} />{title}
-                    </span>
-                    <hr />
-                </div>
-                {this.state.expanded ? this.props.children : null}
-            </>);
-    }
+    const cls = "expander-caret fa " + (expanded ? "fa-angle-down" : "fa-angle-right");
+    return (
+        <>
+            <div className="expander-title">
+                <hr />
+                <Button variant="link" onClick={ () => setExpanded(!expanded) }>
+                    <i className={cls} />{title}
+                </Button>
+                <hr />
+            </div>
+            {expanded ? children : null}
+        </>);
 }
 
 function count_security_updates(updates) {
@@ -142,6 +141,14 @@ function count_security_updates(updates) {
     return num_security;
 }
 
+function find_highest_severity(updates) {
+    var max = PK.Enum.INFO_LOW;
+    for (const u in updates)
+        if (updates[u].severity > max)
+            max = updates[u].severity;
+    return max;
+}
+
 function HeaderBar(props) {
     var num_updates = Object.keys(props.updates).length;
     var num_security = 0;
@@ -172,11 +179,11 @@ function HeaderBar(props) {
     var actionButton;
     if (props.state == "uptodate" || props.state == "available") {
         if (!props.unregistered)
-            actionButton = <button className="btn btn-default" onClick={props.onRefresh}>{_("Check for Updates")}</button>;
+            actionButton = <Button variant="secondary" onClick={props.onRefresh}>{_("Check for Updates")}</Button>;
         if (props.timeSinceRefresh !== null)
-            lastChecked = cockpit.format(_("Last checked: $0 ago"), moment.duration(props.timeSinceRefresh * 1000).humanize());
+            lastChecked = cockpit.format(_("Last checked: $0"), moment(moment().valueOf() - props.timeSinceRefresh * 1000).fromNow());
     } else if (props.state == "applying") {
-        actionButton = <button className="btn btn-default" onClick={props.onCancel} disabled={!props.allowCancel}>{_("Cancel")}</button>;
+        actionButton = <Button variant="secondary" onClick={props.onCancel} isDisabled={!props.allowCancel}>{_("Cancel")}</Button>;
     }
 
     return (
@@ -214,6 +221,7 @@ class UpdateItem extends React.Component {
     constructor() {
         super();
         this.state = { expanded: false };
+        this.remarkable = new Remarkable();
     }
 
     render() {
@@ -223,7 +231,7 @@ class UpdateItem extends React.Component {
         if (info.bug_urls && info.bug_urls.length) {
             // we assume a bug URL ends with a number; if not, show the complete URL
             bugs = insertCommas(info.bug_urls.map(url => (
-                <a key={url} rel="noopener" referrerPolicy="no-referrer" target="_blank" href={url}>
+                <a key={url} rel="noopener noreferrer" target="_blank" href={url}>
                     {url.match(/[0-9]+$/) || url}
                 </a>)
             ));
@@ -232,7 +240,7 @@ class UpdateItem extends React.Component {
         var cves = null;
         if (info.cve_urls && info.cve_urls.length) {
             cves = insertCommas(info.cve_urls.map(url => (
-                <a key={url} href={url} rel="noopener" referrerPolicy="no-referrer" target="_blank">
+                <a key={url} href={url} rel="noopener noreferrer" target="_blank">
                     {url.match(/[^/=]+$/)}
                 </a>)
             ));
@@ -241,7 +249,7 @@ class UpdateItem extends React.Component {
         var errata = null;
         if (info.vendor_urls) {
             errata = insertCommas(info.vendor_urls.filter(url => url.indexOf("/errata/") > 0).map(url => (
-                <a key={url} href={url} rel="noopener" referrerPolicy="no-referrer" target="_blank">
+                <a key={url} href={url} rel="noopener noreferrer" target="_blank">
                     {url.match(/[^/=]+$/)}
                 </a>)
             ));
@@ -255,7 +263,7 @@ class UpdateItem extends React.Component {
         var type;
         if (info.severity === PK.Enum.INFO_SECURITY) {
             if (secSeverityURL)
-                secSeverityURL = <a rel="noopener" referrerPolicy="no-referrer" target="_blank" href={secSeverityURL}>{secSeverity}</a>;
+                secSeverityURL = <a rel="noopener noreferrer" target="_blank" href={secSeverityURL}>{secSeverity}</a>;
             type = (
                 <>
                     <OverlayTrigger overlay={ <Tooltip id="tip-severity">{ secSeverity || _("security") }</Tooltip> } placement="top">
@@ -290,8 +298,8 @@ class UpdateItem extends React.Component {
         descriptionFirstLine = cleanupChangelogLine(descriptionFirstLine);
         var description;
         if (info.markdown) {
-            descriptionFirstLine = <Markdown source={descriptionFirstLine} />;
-            description = <Markdown source={info.description} />;
+            descriptionFirstLine = <span dangerouslySetInnerHTML={{ __html: this.remarkable.render(descriptionFirstLine) }} />;
+            description = <div dangerouslySetInnerHTML={{ __html: this.remarkable.render(info.description) }} />;
         } else {
             description = <div className="changelog">{info.description}</div>;
         }
@@ -488,24 +496,20 @@ class ApplyUpdates extends React.Component {
     }
 }
 
-function AskRestart(props) {
-    return (
-        <div className="blank-slate-pf">
-            <h1>{_("Restart Recommended")}</h1>
-            <p>{_("Updated packages may require a restart to take effect.")}</p>
-            <div className="blank-slate-pf-secondary-action">
-                <button className="btn btn-default" onClick={props.onIgnore}>{_("Ignore")}</button>
-                &nbsp;
-                <button className="btn btn-primary" onClick={props.onRestart}>{_("Restart Now")}</button>
-            </div>
-            <div className="flow-list-blank-slate">
-                <Expander title={_("Package information")}>
-                    <PackageList packages={props.history[0]} />
-                </Expander>
-            </div>
-        </div>
-    );
-}
+const AskRestart = ({ onIgnore, onRestart, history }) => <>
+    <EmptyStatePanel icon={RebootingIcon}
+                     title={ _("Restart Recommended") }
+                     paragraph={ _("Updated packages may require a restart to take effect.") }
+                     action={ _("Restart Now") }
+                     onAction={ onRestart}
+                     secondary={ [<Button key="ignore" variant="secondary" onClick={onIgnore}>{_("Ignore")}</Button>] } />
+
+    <div className="flow-list-blank-slate">
+        <Expander title={_("Package information")}>
+            <PackageList packages={history[0]} />
+        </Expander>
+    </div>
+</>;
 
 class OsUpdates extends React.Component {
     constructor() {
@@ -536,14 +540,11 @@ class OsUpdates extends React.Component {
                     const promises = transactions.map(transactionPath => PK.call(
                         transactionPath, "org.freedesktop.DBus.Properties", "Get", [PK.transactionInterface, "Role"]));
 
-                    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-                    // https://github.com/cockpit-project/cockpit/issues/10956
-                    // eslint-disable-next-line cockpit/no-cockpit-all
-                    cockpit.all(promises)
+                    Promise.all(promises)
                             .then(roles => {
                                 // any transaction with UPDATE_PACKAGES role?
                                 for (let idx = 0; idx < roles.length; ++idx) {
-                                    if (roles[idx].v === PK.Enum.ROLE_UPDATE_PACKAGES) {
+                                    if (roles[idx][0].v === PK.Enum.ROLE_UPDATE_PACKAGES) {
                                         this.watchUpdates(transactions[idx]);
                                         return;
                                     }
@@ -562,7 +563,7 @@ class OsUpdates extends React.Component {
     }
 
     handleLoadError(ex) {
-        console.error("loading available updates failed:", JSON.stringify(ex));
+        console.warn("loading available updates failed:", JSON.stringify(ex));
         if (ex.problem === "not-found")
             ex = _("PackageKit is not installed");
         this.state.errorMessages.push(ex.detail || ex.message || ex);
@@ -682,15 +683,29 @@ class OsUpdates extends React.Component {
     initialLoadOrRefresh() {
         PK.watchRedHatSubscription(registered => this.setState({ unregistered: !registered }));
 
+        cockpit.addEventListener("visibilitychange", () => {
+            if (!cockpit.hidden)
+                this.loadOrRefresh(false);
+        });
+
+        if (!cockpit.hidden)
+            this.loadOrRefresh(true);
+        else
+            this.loadUpdates();
+    }
+
+    loadOrRefresh(always_load) {
         PK.call("/org/freedesktop/PackageKit", "org.freedesktop.PackageKit", "GetTimeSinceAction",
                 [PK.Enum.ROLE_REFRESH_CACHE])
-                .done(seconds => {
+                .done(results => {
+                    const seconds = results[0];
+
                     this.setState({ timeSinceRefresh: seconds });
 
                     // automatically trigger refresh for ≥ 1 day or if never refreshed
                     if (seconds >= 24 * 3600 || seconds < 0)
                         this.handleRefresh();
-                    else
+                    else if (always_load)
                         this.loadUpdates();
                 })
                 .fail(this.handleLoadError);
@@ -759,36 +774,46 @@ class OsUpdates extends React.Component {
                 .catch(ex => {
                     this.state.errorMessages.push(ex.message);
                     this.setState({ state: "updateError" });
-                })
-                // tell the System page to refresh its "Available Updates" status
-                .finally(() => window.sessionStorage.removeItem("last_sw_update_check"));
+                });
     }
 
     renderContent() {
         var applySecurity, applyAll, unregisteredWarning;
 
         if (this.state.unregistered) {
-            // always show empty state pattern, even if there are some repositories enabled that don't require subscriptions
-            return (
-                <div className="blank-slate-pf">
-                    <div className="blank-slate-pf-icon">
-                        <span className="fa fa-exclamation-circle" />
-                    </div>
-                    <h1>{_("This system is not registered")}</h1>
-                    <p>{_("To get software updates, this system needs to be registered with Red Hat, either using the Red Hat Customer Portal or a local subscription server.")}</p>
-                    <div className="blank-slate-pf-main-action">
-                        <button className="btn btn-lg btn-primary"
-                            onClick={ () => cockpit.jump("/subscriptions", cockpit.transport.host) }>
-                            {_("Register…")}
-                        </button>
-                    </div>
-                </div>);
+            // always show empty state pattern, even if there are some
+            // repositories enabled that don't require subscriptions
+
+            page_status.set_own({
+                type: "warning",
+                title: _("Not Registered"),
+                details: {
+                    link: "subscriptions",
+                    icon: "fa fa-exclamation-triangle"
+                }
+            });
+
+            return <EmptyStatePanel
+                title={_("This system is not registered")}
+                paragraph={ _("To get software updates, this system needs to be registered with Red Hat, either using the Red Hat Customer Portal or a local subscription server.") }
+                icon={ExclamationCircleIcon}
+                action={ _("Register…") }
+                onAction={ () => cockpit.jump("/subscriptions", cockpit.transport.host) } />;
         }
 
         switch (this.state.state) {
         case "loading":
         case "refreshing":
         case "locked":
+            page_status.set_own({
+                type: null,
+                title: _("Checking for package updates..."),
+                details: {
+                    link: false,
+                    icon: "spinner spinner-xs",
+                }
+            });
+
             if (this.state.loadPercent)
                 return (
                     <div className="progress-main-view">
@@ -798,25 +823,44 @@ class OsUpdates extends React.Component {
                     </div>
                 );
             else
-                return <div className="spinner spinner-lg progress-main-view" />;
+                return <EmptyStatePanel loading />;
 
         case "available":
             {
                 const num_updates = Object.keys(this.state.updates).length;
                 const num_security_updates = count_security_updates(this.state.updates);
+                const highest_severity = find_highest_severity(this.state.updates);
+                let text;
 
                 applyAll = (
-                    <button className="pk-update--all btn btn-primary" onClick={ () => this.applyUpdates(false) }>
+                    <Button variant="primary" className="pk-update--all" onClick={ () => this.applyUpdates(false) }>
                         { num_updates == num_security_updates
                             ? _("Install Security Updates") : _("Install All Updates") }
-                    </button>);
+                    </Button>);
 
                 if (num_security_updates > 0 && num_updates > num_security_updates) {
                     applySecurity = (
-                        <button className="pk-update--security btn btn-default" onClick={ () => this.applyUpdates(true) }>
+                        <Button variant="secondary" className="pk-update--security" onClick={ () => this.applyUpdates(true) }>
                             {_("Install Security Updates")}
-                        </button>);
+                        </Button>);
                 }
+
+                if (highest_severity == PK.Enum.INFO_SECURITY)
+                    text = _("Security Updates Available");
+                else if (highest_severity >= PK.Enum.INFO_NORMAL)
+                    text = _("Bug Fix Updates Available");
+                else if (highest_severity >= PK.Enum.INFO_LOW)
+                    text = _("Enhancement Updates Available");
+                else
+                    text = _("Updates Available");
+
+                page_status.set_own({
+                    type: num_security_updates > 0 ? "warning" : "info",
+                    title: text,
+                    details: {
+                        icon: PK.getSeverityIcon(highest_severity)
+                    }
+                });
             }
 
             return (
@@ -852,34 +896,45 @@ class OsUpdates extends React.Component {
 
         case "loadError":
         case "updateError":
+            page_status.set_own({
+                type: "error",
+                title: STATE_HEADINGS[this.state.state],
+                details: {
+                    icon: "fa fa-exclamation-circle"
+                }
+            });
             return this.state.errorMessages.map(m => <pre key={m}>{m}</pre>);
 
         case "applying":
+            page_status.set_own(null);
             return <ApplyUpdates transaction={this.state.applyTransaction} />;
 
         case "updateSuccess":
+            page_status.set_own({
+                type: "warning",
+                title: _("Restart Recommended")
+            });
+
             return <AskRestart onRestart={this.handleRestart} onIgnore={this.loadUpdates} history={this.state.history} />;
 
         case "restart":
-            return (
-                <div className="blank-slate-pf">
-                    <div className="blank-slate-pf-icon">
-                        <div className="spinner spinner-lg" />
-                    </div>
-                    <h1>{_("Restarting")}</h1>
-                    <p>{_("Your server will close the connection soon. You can reconnect after it has restarted.")}</p>
-                </div>);
+            page_status.set_own(null);
+            return <EmptyStatePanel loading title={ _("Restarting") }
+                                    paragraph={ _("Your server will close the connection soon. You can reconnect after it has restarted.") } />;
 
         case "uptodate":
+            page_status.set_own({
+                title: _("System is up to date"),
+                details: {
+                    link: false,
+                    icon: "fa fa-check-circle-o"
+                }
+            });
+
             return (
                 <>
                     <AutoUpdates onInitialized={ enabled => this.setState({ autoUpdatesEnabled: enabled }) } />
-                    <div className="blank-slate-pf">
-                        <div className="blank-slate-pf-icon">
-                            <span className="fa fa-check" />
-                        </div>
-                        <p>{_("System is up to date")}</p>
-                    </div>
+                    <EmptyStatePanel icon={CheckIcon} title={ _("System is up to date") } />
 
                     { // automatic updates are not tracked by PackageKit, hide history when they are enabled
                         (this.state.autoUpdatesEnabled !== undefined) &&
@@ -888,6 +943,7 @@ class OsUpdates extends React.Component {
                 </>);
 
         default:
+            page_status.set_own(null);
             return null;
         }
     }
@@ -897,8 +953,6 @@ class OsUpdates extends React.Component {
         PK.cancellableTransaction("RefreshCache", [true], data => this.setState({ loadPercent: data.percentage }))
                 .then(() => {
                     this.setState({ timeSinceRefresh: 0 });
-                    // tell the System page to refresh its "Available Updates" status
-                    window.sessionStorage.removeItem("last_sw_update_check");
                     this.loadUpdates();
                 })
                 .catch(this.handleLoadError);
@@ -918,7 +972,7 @@ class OsUpdates extends React.Component {
 
     render() {
         return (
-            <div>
+            <>
                 <HeaderBar state={this.state.state} updates={this.state.updates}
                            timeSinceRefresh={this.state.timeSinceRefresh} onRefresh={this.handleRefresh}
                            unregistered={this.state.unregistered}
@@ -927,7 +981,7 @@ class OsUpdates extends React.Component {
                 <div className="container-fluid">
                     {this.renderContent()}
                 </div>
-            </div>
+            </>
         );
     }
 }
