@@ -24,31 +24,24 @@ from time import sleep
 import argparse
 import base64
 import errno
-import subprocess
 import os
-import select
 import shutil
 import socket
 import sys
 import traceback
-import random
 import re
 import json
 import tempfile
 import time
 import unittest
 import gzip
+import inspect
 
-import tap
-try:
-    from machine_core import testvm
-except ImportError:
-    import testvm
+import testvm
 import cdp
 
 TEST_DIR = os.path.normpath(os.path.dirname(os.path.realpath(os.path.join(__file__, ".."))))
 BOTS_DIR = os.path.normpath(os.path.join(TEST_DIR, "..", "bots"))
-_PY3 = sys.version_info[0] >= 3
 
 os.environ["PATH"] = "{0}:{1}:{2}".format(os.environ.get("PATH"), BOTS_DIR, TEST_DIR)
 
@@ -58,10 +51,13 @@ __all__ = (
     'arg_parser',
     'Browser',
     'MachineCase',
+    'nondestructive',
     'skipImage',
+    'skipBrowser',
     'allowImage',
     'skipPackage',
     'enableAxe',
+    'timeout',
     'Error',
 
     'sit',
@@ -213,6 +209,9 @@ class Browser:
     def click(self, selector):
         self.mouse(selector + ":not([disabled])", "click", 0, 0, 0)
 
+    def mousedown(self, selector):
+        self.mouse(selector + ":not([disabled])", "mousedown", 0, 0, 0)
+
     def val(self, selector):
         self.wait_visible(selector)
         return self.call_js_func('ph_val', selector)
@@ -233,6 +232,10 @@ class Browser:
         self.wait_present(selector + ':not([disabled])')
         self.call_js_func('ph_set_attr', selector, attr, val and 'true' or 'false')
 
+    def get_checked(self, selector):
+        self.wait_visible(selector + ':not([disabled])')
+        return self.call_js_func('ph_get_checked', selector)
+
     def set_checked(self, selector, val):
         self.wait_visible(selector + ':not([disabled])')
         self.call_js_func('ph_set_checked', selector, val)
@@ -245,19 +248,51 @@ class Browser:
         self.wait_visible(selector + ':not([disabled])')
         self.call_js_func('ph_blur', selector)
 
-    def key_press(self, keys, modifiers=0):
+    # TODO: Unify them so we can have only one
+    def key_press(self, keys, modifiers=0, use_ord=False):
+        if self.cdp.browser == "chromium":
+            self.key_press_chromium(keys, modifiers, use_ord)
+        else:
+            self.key_press_firefox(keys, modifiers, use_ord)
+
+    def key_press_chromium(self, keys, modifiers=0, use_ord=False):
         for key in keys:
             args = {"type": "keyDown", "modifiers": modifiers}
 
             # If modifiers are used we need to pass windowsVirtualKeyCode which is
             # basically the asci decimal representation of the key
             args["text"] = key
-            if (not key.isalnum() and ord(key) < 32) or modifiers != 0:
+            if use_ord:
+                args["windowsVirtualKeyCode"] = ord(key)
+            elif (not key.isalnum() and ord(key) < 32) or modifiers != 0:
                 args["windowsVirtualKeyCode"] = ord(key.upper())
             else:
                 args["key"] = key
 
             self.cdp.invoke("Input.dispatchKeyEvent", **args)
+            args["type"] = "keyUp"
+            self.cdp.invoke("Input.dispatchKeyEvent", **args)
+
+    def key_press_firefox(self, keys, modifiers=0, use_ord=False):
+        # https://github.com/GoogleChrome/puppeteer/blob/master/lib/USKeyboardLayout.js
+        keyMap = {
+            8: "Backspace",  # Backspace key
+            9: "Tab",        # Tab key
+            13: "Enter",     # Enter key
+            27: "Escape",    # Escape key
+            40: "ArrowDown", # Arrow key down
+            45: "Insert",    # Insert key
+        }
+        for key in keys:
+            args = {"type": "keyDown", "modifiers": modifiers}
+
+            args["key"] = key
+            if ord(key) < 32 or use_ord:
+                args["key"] = keyMap[ord(key)]
+
+            self.cdp.invoke("Input.dispatchKeyEvent", **args)
+            args["type"] = "keyUp"
+            self.cdp.invoke("Input.dispatchKeyEvent", **args)
 
     def select_from_dropdown(self, selector, value, substring=False):
         # This is a backwards compat helper method; new code should use .set_val()
@@ -282,7 +317,6 @@ class Browser:
 
         if value_check:
             self.wait_val(selector, val)
-        self.blur(selector)
 
     def set_file_autocomplete_val(self, identifier, location):
         file_item_selector_template = "#{0} li a:contains({1})"
@@ -415,7 +449,7 @@ class Browser:
         """
         self.wait_not_visible('#' + id)
 
-    def dialog_complete(self, sel, button=".btn-primary", result="hide"):
+    def dialog_complete(self, sel, button=".pf-m-primary", result="hide"):
         self.click(sel + " " + button)
         self.wait_not_present(sel + " .dialog-wait-ct")
 
@@ -432,7 +466,7 @@ class Browser:
         else:
             raise Error("invalid dialog result argument: " + result)
 
-    def dialog_cancel(self, sel, button=".btn[data-dismiss='modal']"):
+    def dialog_cancel(self, sel, button="button[data-dismiss='modal']"):
         self.click(sel + " " + button)
         self.wait_not_visible(sel)
 
@@ -485,9 +519,11 @@ class Browser:
         else:
             self.click(sel + ' button:first-child')
 
-    def login_and_go(self, path=None, user=None, host=None, authorized=True, urlroot=None, tls=False):
+    def login_and_go(self, path=None, user=None, host=None, authorized=True, urlroot=None, tls=False, password=None):
         if user is None:
             user = self.default_user
+        if password is None:
+            password = self.password
         href = path
         if not href:
             href = "/"
@@ -498,7 +534,7 @@ class Browser:
         self.open(href, tls=tls)
         self.wait_visible("#login")
         self.set_val('#login-user-input', user)
-        self.set_val('#login-password-input', self.password)
+        self.set_val('#login-password-input', password)
         self.set_checked('#authorized-input', authorized)
 
         self.click('#login-button')
@@ -564,14 +600,21 @@ class Browser:
             self.cdp.command("clearExceptions()")
 
             filename = "{0}-{1}.png".format(label or self.label, title)
-            ret = self.cdp.invoke("Page.captureScreenshot", no_trace=True)
-            if "data" in ret:
-                with open(filename, 'wb') as f:
-                    f.write(base64.standard_b64decode(ret["data"]))
-                attach(filename)
-                print("Wrote screenshot to " + filename)
-            else:
-                print("Screenshot not available")
+            if self.cdp.browser == "chromium":
+                ret = self.cdp.invoke("Page.captureScreenshot", no_trace=True)
+                if "data" in ret:
+                    with open(filename, 'wb') as f:
+                        f.write(base64.standard_b64decode(ret["data"]))
+                    attach(filename)
+                    print("Wrote screenshot to " + filename)
+                else:
+                    print("Screenshot not available")
+            elif self.cdp.browser == "firefox":
+                # API not yet supported
+                # https://bugzilla.mozilla.org/show_bug.cgi?id=1549466
+                # TODO: Possible workaround could be something like:
+                # Runtime.execute(':screenshot --file <path>)
+                pass
 
             filename = "{0}-{1}.html".format(label or self.label, title)
             html = self.cdp.invoke("Runtime.evaluate", expression="document.documentElement.outerHTML",
@@ -607,6 +650,7 @@ class MachineCase(unittest.TestCase):
     image = testvm.DEFAULT_IMAGE
     runner = None
     machine = None
+    global_machine = None
     machines = {}
     machine_class = None
     browser = None
@@ -618,15 +662,29 @@ class MachineCase(unittest.TestCase):
     # These will be instantiated during setUp, and replaced with machine objects
     provision = None
 
+    @classmethod
+    def get_global_machine(klass):
+        if not klass.global_machine:
+            klass.global_machine = klass.new_machine(klass, cleanup=False)
+            if opts.trace:
+                print("Starting global machine {0}".format(klass.global_machine.label))
+            klass.global_machine.start()
+        return klass.global_machine
+
+    @classmethod
+    def kill_global_machine(klass):
+        if klass.network:
+            klass.network.kill()
+            klass.network = None
+        if klass.global_machine:
+            klass.global_machine.kill()
+            klass.global_machine = None
+
     def label(self):
         (unused, sep, label) = self.id().partition(".")
         return label.replace(".", "-")
 
-    def new_machine(self, image=None, forward={}, **kwargs):
-        try:
-            from machine_core import testvm
-        except ImportError:
-            import testvm
+    def new_machine(self, image=None, forward={}, restrict=True, cleanup=True, **kwargs):
         machine_class = self.machine_class
         if image is None:
             image = self.image
@@ -634,19 +692,22 @@ class MachineCase(unittest.TestCase):
             if machine_class or forward:
                 raise unittest.SkipTest("Cannot run this test when specific machine address is specified")
             machine = testvm.Machine(address=opts.address, image=image, verbose=opts.trace, browser=opts.browser)
-            self.addCleanup(lambda: machine.disconnect())
+            if cleanup:
+                self.addCleanup(machine.disconnect)
         else:
             if not machine_class:
                 machine_class = testvm.VirtMachine
             if not self.network:
                 network = testvm.VirtNetwork(image=image)
-                self.addCleanup(lambda: network.kill())
+                if cleanup:
+                    self.addCleanup(network.kill)
                 self.network = network
-            networking = self.network.host(restrict=True, forward=forward)
+            networking = self.network.host(restrict=restrict, forward=forward)
             machine = machine_class(verbose=opts.trace, networking=networking, image=image, **kwargs)
             if opts.fetch and not os.path.exists(machine.image_file):
                 machine.pull(machine.image_file)
-            self.addCleanup(lambda: machine.kill())
+            if cleanup:
+                self.addCleanup(machine.kill)
         return machine
 
     def new_browser(self, machine=None):
@@ -654,11 +715,11 @@ class MachineCase(unittest.TestCase):
             machine = self.machine
         label = self.label() + "-" + machine.label
         browser = Browser(machine.web_address, label=label, port=machine.web_port)
-        self.addCleanup(lambda: browser.kill())
+        self.addCleanup(browser.kill)
         return browser
 
     def checkSuccess(self):
-        if _PY3 and self._outcome:
+        if self._outcome:
             # errors is a list of (method, exception) calls (usually multiple
             # per method); None exception means success
             return not any(e[1] for e in self._outcome.errors)
@@ -680,16 +741,6 @@ class MachineCase(unittest.TestCase):
         return True
 
     def run(self, result=None):
-        if not _PY3:
-            orig_result = result
-
-            # We need a result to intercept, so create one here
-            if result is None:
-                result = self.defaultTestResult()
-                startTestRun = getattr(result, 'startTestRun', None)
-                if startTestRun is not None:
-                    startTestRun()
-
         self.currentResult = result
 
         # Here's the loop to actually retry running the test. It's an awkward
@@ -701,7 +752,7 @@ class MachineCase(unittest.TestCase):
         max_retry_hard_limit = 10
         for retry in range(0, max_retry_hard_limit):
             try:
-                super(MachineCase, self).run(result)
+                super().run(result)
             except RetryError as ex:
                 assert retry < max_retry_hard_limit
                 sys.stderr.write("{0}\n".format(ex))
@@ -711,47 +762,46 @@ class MachineCase(unittest.TestCase):
 
         self.currentResult = None
 
-        if not _PY3:
-            # Standard book keeping that we have to do
-            if orig_result is None:
-                stopTestRun = getattr(result, 'stopTestRun', None)
-                if stopTestRun is not None:
-                    stopTestRun()
-
     def setUp(self):
         if opts.address and self.provision is not None:
             raise unittest.SkipTest("Cannot provision multiple machines if a specific machine address is specified")
 
-        self.machine = None
         self.browser = None
         self.machines = {}
         provision = self.provision or {'machine1': {}}
 
-        # First create all machines, wait for them later
-        for key in sorted(provision.keys()):
-            options = provision[key].copy()
-            if 'address' in options:
-                del options['address']
-            if 'dns' in options:
-                del options['dns']
-            if 'dhcp' in options:
-                del options['dhcp']
-            machine = self.new_machine(**options)
-            self.machines[key] = machine
-            if not self.machine:
-                self.machine = machine
-            if opts.trace:
-                print("Starting {0} {1}".format(key, machine.label))
-            machine.start()
+        test_method = getattr(self.__class__, self._testMethodName)
+        if getattr(test_method, "_testlib__non_destructive", False) and not opts.address:
+            if self.provision:
+                raise unittest.SkipTest("Cannot provision machines if test is marked as nondestructive")
+            self.machine = self.machines['machine1'] = MachineCase.get_global_machine()
+        else:
+            self.machine = None
+            # First create all machines, wait for them later
+            for key in sorted(provision.keys()):
+                options = provision[key].copy()
+                if 'address' in options:
+                    del options['address']
+                if 'dns' in options:
+                    del options['dns']
+                if 'dhcp' in options:
+                    del options['dhcp']
+                machine = self.new_machine(**options)
+                self.machines[key] = machine
+                if not self.machine:
+                    self.machine = machine
+                if opts.trace:
+                    print("Starting {0} {1}".format(key, machine.label))
+                machine.start()
 
-        def sitter():
-            if opts.sit and not self.checkSuccess():
-                if _PY3 and self._outcome:
-                    [traceback.print_exception(*e[1]) for e in self._outcome.errors if e[1]]
-                else:
-                    self.currentResult.printErrors()
-                sit(self.machines)
-        self.addCleanup(sitter)
+        if self.machine.image == "rhel-8-2-distropkg":
+            self.danger_btn_class = '.btn-danger'
+            self.primary_btn_class = '.btn-primary'
+            self.default_btn_class = '.btn-default'
+        else:
+            self.danger_btn_class = '.pf-m-danger'
+            self.primary_btn_class = '.pf-m-primary'
+            self.default_btn_class = '.pf-m-secondary'
 
         # Now wait for the other machines to be up
         for key in self.machines.keys():
@@ -770,8 +820,20 @@ class MachineCase(unittest.TestCase):
         if self.machine:
             self.journal_start = self.machine.journal_cursor()
             self.browser = self.new_browser()
+            if getattr(test_method, "_testlib__non_destructive", False):
+                self.nonDestructiveSetup()
+
         self.tmpdir = tempfile.mkdtemp()
 
+        def sitter():
+            if opts.sit and not self.checkSuccess():
+                if self._outcome:
+                    [traceback.print_exception(*e[1]) for e in self._outcome.errors if e[1]]
+                else:
+                    self.currentResult.printErrors()
+                sit(self.machines)
+        self.addCleanup(sitter)
+
         def intercept():
             if not self.checkSuccess():
                 self.snapshot("FAIL")
@@ -780,6 +842,48 @@ class MachineCase(unittest.TestCase):
                 self.copy_cores("FAIL")
         self.addCleanup(intercept)
 
+    def nonDestructiveSetup(self):
+        '''generic setUp/tearDown for @nondestructive tests'''
+
+        m = self.machine
+
+        # users/groups/home dirs
+        m.execute("cp -a /etc/passwd /etc/passwd.test && "
+                  "cp -a /etc/shadow /etc/shadow.test && "
+                  "cp -a /etc/group /etc/group.test")
+        self.addCleanup(m.execute, "mv /etc/passwd.test /etc/passwd && "
+                                   "mv /etc/shadow.test /etc/shadow && "
+                                   "mv /etc/group.test /etc/group")
+        home_dirs = m.execute("ls /home").strip().split()
+
+        def cleanup_home_dirs():
+            for d in m.execute("ls /home").strip().split():
+                if d not in home_dirs:
+                    m.execute("rm -rf /home/" + d)
+        self.addCleanup(cleanup_home_dirs)
+
+        # cockpit configuration
+        self.addCleanup(m.execute, "rm -f /etc/cockpit/cockpit.conf")
+
+        if not m.ostree_image:
+            # for storage tests
+            m.execute("cp -a /etc/fstab /etc/fstab.test && "
+                      "cp -a /etc/crypttab /etc/crypttab.test")
+            self.addCleanup(m.execute, "mv /etc/fstab.test /etc/fstab && "
+                                       "mv /etc/crypttab.test /etc/crypttab")
+
+            # tests expect cockpit.service to not run at start; also, avoid log leakage into the next test
+            self.addCleanup(m.execute, "systemctl stop cockpit")
+
+        # reset scsi_debug (see e. g. StorageHelpers.add_ram_disk()
+        # this needs to happen very late in the cleanup, so that test cases can clean up the users of that disk first
+        # right after unmounting the device is often still busy, so retry a few times
+        self.addCleanup(self.machine.execute,
+                        "set -e; [ -e /sys/module/scsi_debug ] || exit 0; "
+                        "for dev in $(ls /sys/bus/pseudo/drivers/scsi_debug/adapter*/host*/target*/*:*/block); do "
+                        "   umount /dev/$dev 2>/dev/null || true; "
+                        "done; until rmmod scsi_debug; do sleep 1; done")
+
     def tearDown(self):
         if self.checkSuccess() and self.machine.ssh_reachable:
             self.check_journal_messages()
@@ -822,6 +926,9 @@ class MachineCase(unittest.TestCase):
         # ssh messages may be dropped when closing
         '10.*: dropping message while waiting for child to exit',
 
+        # pkg/packagekit/autoupdates.jsx backend check often gets interrupted by logout
+        "xargs: basename: terminated by signal 13",
+
         # SELinux messages to ignore
         "(audit: )?type=1403 audit.*",
         "(audit: )?type=1404 audit.*",
@@ -833,6 +940,9 @@ class MachineCase(unittest.TestCase):
         "(audit: )?type=1400 .*denied.*comm=\"ssh-transport-c\".*name=\"unix\".*dev=\"proc\".*",
         "(audit: )?type=1400 .*denied.*comm=\"cockpit-ssh\".*name=\"unix\".*dev=\"proc\".*",
 
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1795524
+        "(audit: )?type=1400 .*denied  { setsched } .* comm=\"cockpit-ws\".*",
+
         # apparmor loading
         "(audit: )?type=1400.*apparmor=\"STATUS\".*",
 
@@ -906,6 +1016,8 @@ class MachineCase(unittest.TestCase):
                                     "user user was reauthorized",
                                     "sudo: no password was provided",
                                     "sudo: unable to resolve host .*",
+                                    "sudo: unable to open /run/sudo/ts/unpriv: Permission denied",
+                                    "sudo: unable to stat /var/db/sudo: Permission denied",
                                     ".*: sorry, you must have a tty to run sudo",
                                     ".*/pkexec: bridge exited",
                                     "We trust you have received the usual lecture from the local System",
@@ -928,10 +1040,23 @@ class MachineCase(unittest.TestCase):
         if "TEST_AUDIT_NO_SELINUX" not in os.environ:
             messages += machine.audit_messages("14", cursor=cursor) # 14xx is selinux
 
-        if self.image in ['fedora-30', 'fedora-testing', 'fedora-i386']:
-            # Fedora 30 switched to dbus-broker
+        if self.image in ['fedora-32', 'fedora-31', 'fedora-30', 'fedora-testing']:
+            # Fedora >= 30 switched to dbus-broker
             self.allowed_messages.append("dbus-daemon didn't send us a dbus address; not installed?.*")
 
+        if self.image in ['rhel-8-2', 'rhel-8-2-distropkg']:
+            # HACK: https://bugzilla.redhat.com/show_bug.cgi?id=1753991
+            self.allowed_messages.append('.*type=1400.*avc:  denied  { dac_override } .* comm="rhsmd" .* scontext=system_u:system_r:rhsmcertd_t:s0-s0:c0.c1023 tcontext=system_u:system_r:rhsmcertd_t:.*')
+
+        if self.image in ['debian-testing']:
+            # HACK: https://bugs.debian.org/951477
+            self.allowed_messages.append('Process .* \(ip6?tables\) of user 0 dumped core.*')
+            self.allowed_messages.append('Process .* \(ebtables\) of user 0 dumped core.*')
+            # don't ignore all stack traces
+            self.allowed_messages.append('^#[0-9]+ .*(nftnl|xtables-nft|__libc_start_main).*')
+            # but we have to ignore that general header line
+            self.allowed_messages.append('^Stack trace of.*')
+
         all_found = True
         first = None
         for m in messages:
@@ -949,14 +1074,13 @@ class MachineCase(unittest.TestCase):
             if not found:
                 all_found = False
                 if not first:
-                    print("Unexpected journal messages:")
                     first = m
                 print(m)
         if not all_found:
             self.copy_js_log("FAIL")
             self.copy_journal("FAIL")
             self.copy_cores("FAIL")
-            raise Error(first)
+            raise Error("FAIL: Test completed, but found unexpected journal messages:\n" + first)
 
     def allow_browser_errors(self, *patterns):
         """Don't fail if the test caused a console error contains the given regexp"""
@@ -987,6 +1111,10 @@ class MachineCase(unittest.TestCase):
         # only run this on the default OS test, that's enough
         if os.getenv("TEST_OS") not in [None, testvm.TEST_OS_DEFAULT]:
             return
+        # HACK: We cannot test axe on Firefox since `axe.run()` returns promise
+        # and Firefox CDP cannot wait for it to resolve
+        if self.browser.cdp.browser == "firefox":
+            return
 
         report = self.browser.eval_js("axe.run()", no_trace=True)
 
@@ -1064,14 +1192,32 @@ class MachineCase(unittest.TestCase):
                         print("Core dumps downloaded to %s" % (dest))
                         attach(dest)
 
+    def settle_cpu(self):
+        '''Wait until CPU usage in the VM settles down
 
-some_failed = False
+        Wait until the process with the highest CPU usage drops below 20%
+        usage. Wait for up to a minute, then return. There is no error if the
+        CPU stays busy, as usually a test then should just try to run anyway.
+        '''
+        for retry in range(20):
+            # get the CPU percentage of the most busy process
+            busy_proc = self.machine.execute("ps --no-headers -eo pcpu,pid,args | sort -k 1 -n -r | head -n1")
+            if float(busy_proc.split()[0]) < 20.0:
+                break
+            time.sleep(3)
 
 
 def jsquote(str):
     return json.dumps(str)
 
 
+def skipBrowser(reason, *args):
+    browser = os.environ.get("TEST_BROWSER", "chromium")
+    if browser in args:
+        return unittest.skip("{0}: {1}".format(browser, reason))
+    return lambda func: func
+
+
 def skipImage(reason, *args):
     if testvm.DEFAULT_IMAGE in args:
         return unittest.skip("{0}: {1}".format(testvm.DEFAULT_IMAGE, reason))
@@ -1092,6 +1238,23 @@ def skipPackage(*args):
     return lambda func: func
 
 
+def nondestructive(testEntity):
+    """Tests decorated as nondestructive will all run against the same VM
+
+    Can be used on test classes and individual test methods.
+    """
+    def is_test_function(member):
+        return inspect.isfunction(member) and member.__name__.startswith("test")
+    if inspect.isclass(testEntity) and issubclass(testEntity, MachineCase):
+        for test_function in inspect.getmembers(testEntity, is_test_function):
+            test_function[1]._testlib__non_destructive = True
+    elif is_test_function(testEntity):
+        testEntity._testlib__non_destructive = True
+    else:
+        raise Error("The nondestructive decorator can only be used on test classes and test methods")
+    return testEntity
+
+
 def enableAxe(method):
     """Enable aXe accessibility test code injection for this test case"""
 
@@ -1103,226 +1266,100 @@ def enableAxe(method):
         with open(os.path.join(TEST_DIR, "common/axe.js")) as f:
             script = f.read()
         # first method argument is "self", a MachineCase instance
-        args[0].browser.cdp.invoke("Page.addScriptToEvaluateOnLoad", scriptSource=script, no_trace=True)
+        args[0].browser.cdp.invoke("Page.addScriptToEvaluateOnNewDocument", source=script, no_trace=True)
         return method(*args)
 
     return wrapper
 
 
-class TestResult(tap.TapResult):
-    def __init__(self, stream, descriptions, verbosity):
-        self.policy = None
-        super(TestResult, self).__init__(verbosity)
+def timeout(seconds):
+    """Change default test timeout of 600s, for long running tests
 
-    def startTest(self, test):
-        sys.stdout.write("# {0}\n# {1}\n#\n".format('-' * 70, str(test)))
-        sys.stdout.flush()
-        super(TestResult, self).startTest(test)
+    Can be applied to an individual test method or the entire class. This only
+    applies to test/verify/run-tests, not to calling check-* directly.
+    """
+    def wrapper(testEntity):
+        testEntity.__timeout = seconds
+        return testEntity
 
-    def stopTest(self, test):
-        sys.stdout.write("\n")
-        sys.stdout.flush()
-        super(TestResult, self).stopTest(test)
+    return wrapper
 
 
-class OutputBuffer(object):
-    def __init__(self):
-        self.poll = select.poll()
-        self.buffers = {}
-        self.fds = {}
+class TapRunner:
 
-    def drain(self):
-        while self.fds:
-            for p in self.poll.poll(1000):
-                data = os.read(p[0], 1024)
-                if data == b"":
-                    self.poll.unregister(p[0])
-                else:
-                    self.buffers[p[0]] += data
-            else:
-                break
-
-    def push(self, pid, fd):
-        self.poll.register(fd, select.POLLIN)
-        self.fds[pid] = fd
-        self.buffers[fd] = b""
-
-    def pop(self, pid):
-        fd = self.fds.pop(pid)
-        buffer = self.buffers.pop(fd)
-        try:
-            self.poll.unregister(fd)
-        except KeyError:
-            pass
-        while True:
-            data = os.read(fd, 1024)
-            if data == b"":
-                break
-            buffer += data
-        os.close(fd)
-        return buffer
-
-
-class TapRunner(object):
-    resultclass = TestResult
-
-    def __init__(self, verbosity=1, jobs=1, thorough=False):
+    def __init__(self, verbosity=1):
         self.stream = unittest.runner._WritelnDecorator(sys.stderr)
         self.verbosity = verbosity
-        self.thorough = thorough
-        self.jobs = jobs
 
-    def runOne(self, test, offset):
-        result = TestResult(self.stream, False, self.verbosity)
-        result.offset = offset
+    def runOne(self, test):
+        result = unittest.TestResult()
+        print('# ----------------------------------------------------------------------')
+        print('#', test)
         try:
-            test(result)
+            unittest.TestSuite([test]).run(result)
         except KeyboardInterrupt:
-            return False
+            result.addError(test, sys.exc_info())
+            return result
         except Exception:
+            result.addError(test, sys.exc_info())
             sys.stderr.write("Unexpected exception while running {0}\n".format(test))
             sys.stderr.write(traceback.print_exc())
-            return False
+            return result
         else:
             result.printErrors()
-            return result.wasSuccessful()
+
+        if result.skipped:
+            print("# Result {0} skipped: {1}".format(test, result.skipped[0][1]))
+        elif result.wasSuccessful():
+            print("# Result {0} succeeded".format(test))
+        else:
+            for error in result.errors:
+                print(error[1])
+            print("# Result {0} failed".format(test))
+        return result
 
     def run(self, testable):
-        tap.TapResult.plan(testable)
-
         tests = []
 
         # The things to test
         def collapse(test, tests):
-            if test.countTestCases() == 1:
+            if isinstance(test, unittest.TestCase):
                 tests.append(test)
             else:
                 for t in test:
                     collapse(t, tests)
         collapse(testable, tests)
-
-        # Now setup the count we have
-        count = len(tests)
-        for i, test in enumerate(tests):
-            setattr(test, "tapOffset", i)
+        test_count = len(tests)
 
         # For statistics
         start = time.time()
-
-        pids = {}
-        options = 0
-        buffer = None
-        if not self.thorough and self.verbosity <= 1:
-            buffer = OutputBuffer()
-            options = os.WNOHANG
-        failures = {"count": 0}
-
-        def join_some(n):
-            while len(pids) > n:
-                if buffer:
-                    buffer.drain()
-                try:
-                    (pid, code) = os.waitpid(-1, options)
-                except KeyboardInterrupt:
-                    sys.exit(255)
-                if code & 0xff:
-                    failed = 1
-                else:
-                    failed = (code >> 8) & 0xff
-                if pid:
-                    if buffer:
-                        output = buffer.pop(pid)
-                        test = pids[pid]
-                        failed, retry = self.filterOutput(test, failed, output)
-                        if retry:
-                            tests.append(test)
-                    del pids[pid]
-                failures["count"] += failed
-
-        while True:
-            join_some(self.jobs - 1)
-
-            if not tests:
-                join_some(0)
-
-                # See if we inserted more tests
-                if not tests:
-                    break
-
+        failures = 0
+        skips = []
+        while tests:
             # The next test to test
-            test = tests.pop()
-
-            # Fork off a child process for each test
-            if buffer:
-                (rfd, wfd) = os.pipe()
-
-            sys.stdout.flush()
-            sys.stderr.flush()
-            pid = os.fork()
-            if not pid:
-                if buffer:
-                    os.dup2(wfd, 1)
-                    os.dup2(wfd, 2)
-                random.seed()
-                offset = getattr(test, "tapOffset", 0)
-                if self.runOne(test, offset):
-                    sys.exit(0)
-                else:
-                    sys.exit(1)
-
-            # The parent process
-            pids[pid] = test
-            if buffer:
-                os.close(wfd)
-                buffer.push(pid, rfd)
+            test = tests.pop(0)
+            result = self.runOne(test)
+            if not result.wasSuccessful():
+                failures += 1
+            skips += result.skipped
 
         # Report on the results
         duration = int(time.time() - start)
         hostname = socket.gethostname().split(".")[0]
         details = "[{0}s on {1}]".format(duration, hostname)
-        count = failures["count"]
-        if count:
-            sys.stdout.write("# {0} TESTS FAILED {1}\n".format(count, details))
+
+        MachineCase.kill_global_machine()
+
+        # Return 77 if all tests were skipped
+        if len(skips) == test_count:
+            sys.stdout.write("# SKIP {1}\n".format(failures, ", ".join(["{0} {1}".format(str(s[0]), s[1]) for s in skips])))
+            return 77
+        if failures:
+            sys.stdout.write("# {0} TEST{1} FAILED {2}\n".format(failures, "S" if failures > 1 else "", details))
+            return 1
         else:
-            sys.stdout.write("# TESTS PASSED {0}\n".format(details))
-        return count
-
-    def filterOutput(self, test, failed, output):
-        # Check how many retries we can do of this test
-        tries = getattr(test, "retryCount", 0)
-        tries += 1
-        setattr(test, "retryCount", tries)
-        # "output" is bytes, grab corresponding stream
-        out = _PY3 and sys.stdout.buffer or sys.stdout
-
-        # Didn't fail or retried too much, just print output and continue
-        if tries >= 3 or not failed:
-            out.write(output)
-            return failed, False
-
-        # Otherwise pass through this command if it exists
-        cmd = ["tests-policy", testvm.DEFAULT_IMAGE]
-        try:
-            proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
-            (changed, unused) = proc.communicate(output)
-            if proc.returncode == 0:
-                output = changed
-        except OSError as ex:
-            if ex.errno != errno.ENOENT:
-                sys.stderr.write("Couldn't run tests-policy: {0}\n".format(str(ex)))
-
-        # Just retry failures always, we don't need to be precious about flakes
-        if b"# SKIP " not in output:
-            output += b"\n# RETRY \n"
-
-        # Write the output bytes
-        out.write(output)
-
-        if b"# SKIP " in output or b"# RETRY" in output:
-            failed = 0
-
-        # Whether we should retry the test or not
-        return failed, b"# RETRY " in output
+            sys.stdout.write("# {0} TEST{1} PASSED {2}\n".format(test_count, "S" if test_count > 1 else "", details))
+            return 0
 
 
 def print_tests(tests):
@@ -1338,16 +1375,12 @@ def print_tests(tests):
 
 def arg_parser():
     parser = argparse.ArgumentParser(description='Run Cockpit test(s)')
-    parser.add_argument('-j', '--jobs', dest="jobs", type=int,
-                        default=os.environ.get("TEST_JOBS", 1), help="Number of concurrent jobs")
     parser.add_argument('-v', '--verbose', dest="verbosity", action='store_const',
                         const=2, help='Verbose output')
     parser.add_argument('-t', "--trace", dest='trace', action='store_true',
                         help='Trace machine boot and commands')
     parser.add_argument('-q', '--quiet', dest='verbosity', action='store_const',
                         const=0, help='Quiet output')
-    parser.add_argument('--thorough', dest='thorough', action='store_true',
-                        help='Thorough mode, no skipping known issues')
     parser.add_argument('-s', "--sit", dest='sit', action='store_true',
                         help="Sit and wait after test failure")
     parser.add_argument('--nonet', dest="fetch", action="store_false",
@@ -1371,9 +1404,8 @@ def test_main(options=None, suite=None, attachments=None, **kwargs):
 
     # Turn off python stdout buffering
     buf_arg = 0
-    if _PY3:
-        os.environ['PYTHONUNBUFFERED'] = '1'
-        buf_arg = 1
+    os.environ['PYTHONUNBUFFERED'] = '1'
+    buf_arg = 1
     sys.stdout.flush()
     sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', buf_arg)
 
@@ -1395,9 +1427,6 @@ def test_main(options=None, suite=None, attachments=None, **kwargs):
     for (key, value) in vars(options).items():
         setattr(opts, key, value)
 
-    if opts.sit and opts.jobs > 1:
-        parser.error("the -s or --sit argument not avalible with multiple jobs")
-
     opts.address = getattr(opts, "address", None)
     opts.browser = getattr(opts, "browser", None)
     opts.attachments = os.environ.get("TEST_ATTACHMENTS", attachments)
@@ -1416,7 +1445,7 @@ def test_main(options=None, suite=None, attachments=None, **kwargs):
         print_tests(suite)
         return 0
 
-    runner = TapRunner(verbosity=opts.verbosity, jobs=opts.jobs, thorough=opts.thorough)
+    runner = TapRunner(verbosity=opts.verbosity)
     ret = runner.run(suite)
     if not standalone:
         return ret
